package com.juablaz.MyPlaces.Controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.juablaz.MyPlaces.Model.Entity.Place;
import com.juablaz.MyPlaces.Model.Entity.SharedPlace;
import com.juablaz.MyPlaces.Model.Entity.User;
import com.juablaz.MyPlaces.Service.AndroidPushNotificationService;
import com.juablaz.MyPlaces.Service.PlaceService;
import com.juablaz.MyPlaces.Service.UserService;
import com.juablaz.MyPlaces.Util.CustomOKResponse;
import com.juablaz.MyPlaces.Util.CustomResponse;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;

@RestController
public class PlaceController {
    private static final Logger logger = LoggerFactory.getLogger(PlaceController.class);

    @Autowired
    PlaceService placeService;

    @Autowired
    AndroidPushNotificationService androidPushNotificationService;

    @Autowired
    UserService userService;


    ObjectMapper objectMapper = new ObjectMapper();

    // Creación de lugares.
    @RequestMapping(value = "/places", method = RequestMethod.POST)
    ResponseEntity<?> addPlaces(@RequestBody String placeDto) {
        logger.info("[PlaceController][places] Saving places: {}", placeDto);
        ArrayList<Place> places = new ArrayList<Place>();
        User user = new User();
        try {
            JSONArray jsonArray = new JSONArray(placeDto);

            JSONObject userJSON = jsonArray.getJSONObject(0);
            JSONArray markersJSON = jsonArray.getJSONArray(1);
            //convert json string to object
            user = objectMapper.readValue(userJSON.toString(), User.class);


            places = new ArrayList(markersJSON.length());
            for (int i = 0; i < markersJSON.length(); i++) {
                Place place = objectMapper.readValue(markersJSON.get(i).toString(), Place.class);
                places.add(place);
            }


        } catch (Exception e) {
            logger.info("Places {}", e.toString());
        }

        try {
            placeService.savePlaces(places, user);

            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse("Successfully added");
            listaRespuesta.add(respuesta);
            return ResponseEntity.ok().body(listaRespuesta);

        } catch (Exception e) {
            logger.info("[PlaceController][places] Saving places: Duplicated places.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.CONFLICT.value()), String.valueOf(HttpStatus.CONFLICT.getReasonPhrase()), "Unable to save places. User " +
                    user.getUsername() + " places already exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }


    }

    @RequestMapping(value = "/places", method = RequestMethod.DELETE)
    ResponseEntity<?> removePlaces(@RequestBody String placeDto) {
        logger.info("[PlaceController][places] Deleting places: {}", placeDto);

        ArrayList<Place> places = new ArrayList<Place>();
        User user = new User();
        try {
            JSONArray jsonArray = new JSONArray(placeDto);

            JSONObject userJSON = jsonArray.getJSONObject(0);
            JSONArray placesJSON = jsonArray.getJSONArray(1);

            //convert json string to object
            user = objectMapper.readValue(userJSON.toString(), User.class);

            places = new ArrayList(placesJSON.length());
            for (int i = 0; i < placesJSON.length(); i++) {
                Place place = objectMapper.readValue(placesJSON.get(i).toString(), Place.class);
                places.add(place);
            }

        } catch (Exception e) {
            logger.info("Places {}", e.toString());
        }

        try {
            placeService.removePlaces(places, user);
            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse("Successfully deleted");
            listaRespuesta.add(respuesta);
            return ResponseEntity.ok().body(listaRespuesta);


        } catch (Exception e) {
            logger.info("[PlaceController][places] Deleting places: Places not found.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.NO_CONTENT.value()), String.valueOf(HttpStatus.NO_CONTENT.getReasonPhrase()), "Unable to delete places. User " +
                    user.getUsername() + " places dont exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }

    }


    // Obtención de lugares.
    @RequestMapping(value = "/places", method = RequestMethod.GET)
    ResponseEntity<?> getPlaces(@RequestParam String username) {

        logger.info("[PlaceController][places] Getting places");
        logger.info(username);

        try {
            List<Place> places = placeService.getPlaces(username);

            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse(places,"getAll");
            listaRespuesta.add(respuesta);

            return ResponseEntity.ok().body(listaRespuesta);


        } catch (Exception e) {
            logger.info("[PlaceController][places] Getting places: Places not found.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.NO_CONTENT.value()), String.valueOf(HttpStatus.NO_CONTENT.getReasonPhrase()), "Unable to delete places. User " +
                    username + " places dont exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }


    }


    @RequestMapping(value = "/places/place", method = RequestMethod.PUT)
    ResponseEntity<?> editPlaces(@RequestBody String placeDto) {
        logger.info("[PlaceController][places] Updating places: {}", placeDto);

        ArrayList<Place> places = new ArrayList<Place>();
        User user = new User();
        try {
            JSONArray jsonArray = new JSONArray(placeDto);

            JSONObject userJSON = jsonArray.getJSONObject(0);
            JSONArray placesJSON = jsonArray.getJSONArray(1);

            //convert json string to object
            user = objectMapper.readValue(userJSON.toString(), User.class);

            places = new ArrayList(placesJSON.length());
            for (int i = 0; i < placesJSON.length(); i++) {
                Place place = objectMapper.readValue(placesJSON.get(i).toString(), Place.class);
                places.add(place);
            }

        } catch (Exception e) {
            logger.info("Places {}", e.toString());
        }

        try {
            placeService.editPlaces(places, user);
            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse("Successfully updated");
            listaRespuesta.add(respuesta);
            return ResponseEntity.ok().body(listaRespuesta);


        } catch (Exception e) {
            logger.info("[PlaceController][places] Updating places: Places not found.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.NO_CONTENT.value()), String.valueOf(HttpStatus.NO_CONTENT.getReasonPhrase()), "Unable to update places. User " +
                    user.getUsername() + " places dont exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }
    }


    // Creación de lugares.
    @RequestMapping(value = "/places/share", method = RequestMethod.POST)
    ResponseEntity<?> sharePlaces(@RequestBody String placeDto) {
        logger.info("[PlaceController][places] Saving places: {}", placeDto);
        ArrayList<SharedPlace> sharedPlaces = new ArrayList<>();
        User user = new User();
        User targetUser = new User();
        try {
            JSONArray jsonArray = new JSONArray(placeDto);

            JSONObject userJSON = jsonArray.getJSONObject(0);
            JSONObject targetUserJSON = jsonArray.getJSONObject(2);
            JSONArray markersJSON = jsonArray.getJSONArray(1);
            //convert json string to object
            user = objectMapper.readValue(userJSON.toString(), User.class);
            targetUser = userService.getUserByUsername(targetUserJSON.get("target_username").toString());/*objectMapper.readValue(targetUserJSON.toString(), User.class);*/


            sharedPlaces = new ArrayList(markersJSON.length());
            for (int i = 0; i < markersJSON.length(); i++) {
                SharedPlace place = objectMapper.readValue(markersJSON.get(i).toString(), SharedPlace.class);
                sharedPlaces.add(place);
            }


        } catch (Exception e) {
            logger.info("Places {}", e.toString());
        }

        try {
            placeService.sharePlaces(sharedPlaces, user, targetUser);

            JSONObject body = new JSONObject();
            body.put("to", targetUser.getMyFirebaseToken());
            body.put("priority", "high");

            JSONObject notification = new JSONObject();
            notification.put("title", "¡Nuevos lugares disponibles!");

            notification.put("body", targetUser.getUsername()+" tienes nuevos lugares de "+ user.getUsername());
            body.put("notification", notification);


            logger.info(body.toString());

            HttpEntity<String> request = new HttpEntity<>(body.toString());

            CompletableFuture<String> pushNotification = androidPushNotificationService.send(request);
            CompletableFuture.allOf(pushNotification).join();


            String firebaseResponse = pushNotification.get();

            logger.info(firebaseResponse);

            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse("Successfully shared");
            listaRespuesta.add(respuesta);


            return ResponseEntity.ok().body(listaRespuesta);

        } catch (Exception e) {
            logger.info("[PlaceController][places] Sharing places: Not found user.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.CONFLICT.value()), String.valueOf(HttpStatus.CONFLICT.getReasonPhrase()), "Unable to save places. User " +
                    user.getUsername() + " not found.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }


    }



    // ##### LUGARES COMPARTIDOS #####
    // Obtención de lugares.
    @RequestMapping(value = "/shared_places", method = RequestMethod.GET)
    ResponseEntity<?> getSharedPlaces(@RequestParam String username) {

        logger.info("[PlaceController][places] Getting Shared Places");
        logger.info(username);

        try {
            List<SharedPlace> places = placeService.getSharedPlaces(username);

            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse(places,"shared");
            listaRespuesta.add(respuesta);

            return ResponseEntity.ok().body(listaRespuesta);


        } catch (Exception e) {
            logger.info("[PlaceController][places] Getting places: Sharing Places not found.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.NO_CONTENT.value()), String.valueOf(HttpStatus.NO_CONTENT.getReasonPhrase()), "Unable to delete places. User " +
                    username + " places dont exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }


    }

    @RequestMapping(value = "/shared_places", method = RequestMethod.DELETE)
    ResponseEntity<?> removeSharedPlaces(@RequestBody String placeDto) {
        logger.info("[PlaceController][places] Deleting shared places: {}", placeDto);

        ArrayList<SharedPlace> places = new ArrayList<>();
        User user = new User();
        try {
            JSONArray jsonArray = new JSONArray(placeDto);

            JSONObject userJSON = jsonArray.getJSONObject(0);
            JSONArray placesJSON = jsonArray.getJSONArray(1);

            logger.info(placesJSON.get(0).toString());

            //convert json string to object
            user = objectMapper.readValue(userJSON.toString(), User.class);

            places = new ArrayList(placesJSON.length());

            for (int i = 0; i < placesJSON.length(); i++) {
                JSONObject placeJSON = placesJSON.getJSONObject(i);

                SharedPlace sharedPlace = new SharedPlace();
                sharedPlace.setLatitude((Double)placeJSON.get("latitude"));
                sharedPlace.setLongitude((Double)placeJSON.get("longitude"));
                User userSharing = new User();
                userSharing.setUsername(placeJSON.get("user_sharing").toString());
                sharedPlace.setUserSharing(userSharing);

                places.add(sharedPlace);
            }

        } catch (Exception e) {
            logger.info("Places {}", e.toString());
        }

        try {
            placeService.removeSharedPlaces(places, user);
            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse("Successfully deleted shared places");
            listaRespuesta.add(respuesta);
            return ResponseEntity.ok().body(listaRespuesta);


        } catch (Exception e) {
            logger.info("[PlaceController][places] Deleting shared places: Places not found.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.NO_CONTENT.value()), String.valueOf(HttpStatus.NO_CONTENT.getReasonPhrase()), "Unable to delete places. User " +
                    user.getUsername() + " places dont exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }

    }


    // Creación de lugares.
    @RequestMapping(value = "/shared_places", method = RequestMethod.POST)
    ResponseEntity<?> addSharePlaces(@RequestBody String placeDto) {
        logger.info("[PlaceController][places] Adding shared places: {}", placeDto);

        ArrayList places = new ArrayList<>();
        User user = new User();
        try {
            JSONArray jsonArray = new JSONArray(placeDto);

            JSONObject userJSON = jsonArray.getJSONObject(0);
            JSONArray placesJSON = jsonArray.getJSONArray(1);

            logger.info(placesJSON.get(0).toString());

            //convert json string to object
            user = objectMapper.readValue(userJSON.toString(), User.class);

            places = new ArrayList(placesJSON.length());

            for (int i = 0; i < placesJSON.length(); i++) {
                JSONObject placeJSON = placesJSON.getJSONObject(i);

                SharedPlace sharedPlace = new SharedPlace();
                sharedPlace.setLatitude((Double)placeJSON.get("latitude"));
                sharedPlace.setLongitude((Double)placeJSON.get("longitude"));
                User userSharing = new User();
                userSharing.setUsername(placeJSON.get("user_sharing").toString());
                sharedPlace.setUserSharing(userSharing);

                places.add(sharedPlace);
            }

        } catch (Exception e) {
            logger.info("Places {}", e.toString());
        }

        try {
            placeService.addSharedPlaces(places, user);
            List<CustomOKResponse> listaRespuesta = new ArrayList<>();
            CustomOKResponse respuesta = new CustomOKResponse("Shared places successfully added to your places");
            listaRespuesta.add(respuesta);
            return ResponseEntity.ok().body(listaRespuesta);


        } catch (Exception e) {
            logger.info("[PlaceController][places] Deleting shared places: Places not found.");
            List<CustomResponse> listaRespuesta = new ArrayList<>();
            CustomResponse respuesta = new CustomResponse(String.valueOf(HttpStatus.NO_CONTENT.value()), String.valueOf(HttpStatus.NO_CONTENT.getReasonPhrase()), "Unable to add shared places. User " +
                    user.getUsername() + " places dont exists.");
            listaRespuesta.add(respuesta);

            return new ResponseEntity(listaRespuesta, HttpStatus.CONFLICT);
        }


    }

}
