package com.juablaz.MyPlaces.Controller;

import com.juablaz.MyPlaces.Model.Entity.User;
import com.juablaz.MyPlaces.Service.UserService;
import com.juablaz.MyPlaces.Util.CustomOKResponse;
import com.juablaz.MyPlaces.Util.CustomResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class UserController {
    private static final Logger logger = LoggerFactory.getLogger(UserController.class);

    @Autowired
    UserService userService;

    // Creación de usuarios.
    @RequestMapping(value = "/sing-up", method = RequestMethod.POST)
    ResponseEntity<?> singUp(@RequestBody User user) {
        logger.info("[UserController][singUp] Creating User : {}", user);

        if (userService.userExists(user)) {
            logger.error("[ERROR][UserController][singUp] Unable to create. A User with name {} already exist", user.getUsername());
            return new ResponseEntity(new CustomResponse(String.valueOf(HttpStatus.CONFLICT.value()), String.valueOf(HttpStatus.CONFLICT.getReasonPhrase()), "Unable to create an User. Name " +
                    user.getUsername() + " already exist."), HttpStatus.CONFLICT);
        } else {
            userService.saveUser(user);
            return ResponseEntity.ok().body(new CustomOKResponse("Successfully created"));
        }
    }

    @RequestMapping(value = "/log-in", method = RequestMethod.POST)
    ResponseEntity<?> logIn(@RequestBody User user) {
        logger.info("[UserController][logIn] Log-in User : {}", user);

        if (userService.validateUser(user)) {
            return ResponseEntity.ok().body(new CustomOKResponse("OK. Usuario identificado"));
        } else {
            return ResponseEntity.ok().body(new CustomOKResponse("KO. Datos incorrectos"));
        }
    }


    @RequestMapping(value = "/users", method = RequestMethod.PUT)
    ResponseEntity<?> updateToken(@RequestBody User user) {
        logger.info("[UserController][updateToken] Log-in User : {}", user);

        userService.updateToken(user);

        return ResponseEntity.ok().body(new CustomOKResponse("OK. User successfully updated"));


    }
}
