package com.juablaz.MyPlaces.Service;

import com.juablaz.MyPlaces.Model.Entity.Place;
import com.juablaz.MyPlaces.Model.Entity.SharedPlace;
import com.juablaz.MyPlaces.Model.Entity.User;
import com.juablaz.MyPlaces.Repository.PlaceRepository;
import com.juablaz.MyPlaces.Repository.SharedPlaceRepository;
import com.juablaz.MyPlaces.Repository.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("placeService")
public class PlaceServiceImpl implements PlaceService {

    private static final Logger logger = LoggerFactory.getLogger(PlaceServiceImpl.class);
    private final PlaceRepository placeRepository;
    private final SharedPlaceRepository sharedPlaceRepository;
    private final UserRepository userRepository;

    public PlaceServiceImpl(PlaceRepository placeRepository, UserRepository userRepository, SharedPlaceRepository sharedPlaceRepository) {
        this.placeRepository = placeRepository;
        this.userRepository = userRepository;
        this.sharedPlaceRepository = sharedPlaceRepository;
    }

    @Override
    public void savePlaces(ArrayList<Place> places, User user) {
        User u = userRepository.findByUsername(user.getUsername());

        logger.info(u.toString());

        for (Place p : places) {
            p.setUser(u);
        }
        try {
            placeRepository.saveAll(places);
        } catch (Exception e) {
            throw e;
        }
    }

    @Override
    public void removePlaces(ArrayList<Place> places, User user) throws Exception {
        List<Place> removePlaces = new ArrayList<>();

        User u = userRepository.findByUsername(user.getUsername());
        boolean borrar = true;

        for (Place p : places) {
            p.setUser(u);
            Place found = placeRepository.findByLatitudeAndLongitudeAndUserUsername(p.getLatitude(), p.getLongitude(), u.getUsername());
            if (found != null) {
                logger.info("ENCONTRADO: " + found.toString());
                removePlaces.add(found);
            } else {
                borrar = false;
            }
        }

        if (borrar) {
            placeRepository.deleteAll(removePlaces);
        } else {
            throw new Exception();
        }
    }

    @Override
    public List<Place> getPlaces(String username) {
        return placeRepository.findByUserUsername(username);

    }

    @Override
    public void editPlaces(ArrayList<Place> places, User user) {

        for (Place place : places) {
            Place found = placeRepository.findByLatitudeAndLongitudeAndUserUsername(place.getLatitude(), place.getLongitude(), user.getUsername());
            found.setTitle(place.getTitle());
            found.setSnippet(place.getSnippet());
            placeRepository.save(found);
        }
    }

    @Override
    public void sharePlaces(ArrayList<SharedPlace> places, User user, User targetUser) {
        User u = userRepository.findByUsername(user.getUsername());
        User tu = userRepository.findByUsername(targetUser.getUsername());

        ArrayList sharedPlaces = new ArrayList();
        for (SharedPlace p : places) {
            p.setTargetUser(tu);
            p.setUserSharing(u);
            p.setDate(new Date());
            SharedPlace found_sp = sharedPlaceRepository.findByLatitudeAndLongitudeAndTargetUserUsername(p.getLatitude(), p.getLongitude(), tu.getUsername());

            if (found_sp != null) {
                found_sp.setDate(new Date());
                found_sp.setTitle(p.getTitle());
                found_sp.setSnippet(p.getSnippet());
                sharedPlaces.add(found_sp);
                logger.info("found_sp: " + found_sp.toString());

            }

            if (found_sp == null) {
                sharedPlaces.add(p);
            }

        }


        try {
            sharedPlaceRepository.saveAll(sharedPlaces);
        } catch (Exception e) {
            throw e;
        }
    }

    @Override
    public List<SharedPlace> getSharedPlaces(String username) {
        return sharedPlaceRepository.findByTargetUserUsernameOrderByDateDesc(username);
    }

    @Override
    public void removeSharedPlaces(ArrayList<SharedPlace> places, User user) throws Exception {
        List<SharedPlace> removePlaces = new ArrayList<>();

        User u = userRepository.findByUsername(user.getUsername());
        boolean borrar = true;

        for (SharedPlace p : places) {
            p.setTargetUser(u);
            SharedPlace found = sharedPlaceRepository.findByLatitudeAndLongitudeAndTargetUserUsernameAndUserSharingUsername(p.getLatitude(), p.getLongitude(), u.getUsername(), p.getUserSharing().getUsername());
            if (found != null) {
                logger.info("ENCONTRADO: " + found.toString());
                removePlaces.add(found);
            } else {
                borrar = false;
            }
        }

        if (borrar) {
            sharedPlaceRepository.deleteAll(removePlaces);
        } else {
            throw new Exception();
        }
    }

    @Override
    public void addSharedPlaces(ArrayList<SharedPlace> places, User user) throws Exception {
        List<Place> addedPlaces = new ArrayList<>();
        List<Place> addedEqualPlaces = new ArrayList<>();
        User u = userRepository.findByUsername(user.getUsername());


        for(SharedPlace p : places){
            SharedPlace found = sharedPlaceRepository.findByLatitudeAndLongitudeAndTargetUserUsernameAndUserSharingUsername(p.getLatitude(), p.getLongitude(), u.getUsername(), p.getUserSharing().getUsername());
            Place equalPlacefound = placeRepository.findByLatitudeAndLongitudeAndUserUsername(p.getLatitude(), p.getLongitude(), u.getUsername());
            if (found != null) {
                logger.info("ENCONTRADO: " + found.toString());
                Place place = new Place();
                place.setUser(u);
                place.setSnippet(found.getSnippet());
                place.setTitle(found.getTitle());
                place.setLatitude(found.getLatitude());
                place.setLongitude(found.getLongitude());
                logger.info("Añado: "+place.toString());

                addedPlaces.add(place);
            }

            if(equalPlacefound!=null){
                addedEqualPlaces.add(equalPlacefound);
            }


        }

        if(addedEqualPlaces!=null){
            placeRepository.deleteAll(addedEqualPlaces);
        }

        logger.info("Los que voy a añadir: "+addedPlaces.size());
        placeRepository.saveAll(addedPlaces);

        removeSharedPlaces(places, u);

    }

}
