package com.juablaz.myplaces;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.SeekBar;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.example.juancarlos.myplaces.R;
import com.google.firebase.iid.FirebaseInstanceId;

import java.io.IOException;
import java.util.ArrayList;

import petrov.kristiyan.colorpicker.ColorPicker;

import static com.juablaz.myplaces.CoreMapActivity.colorChanged;
import static com.juablaz.myplaces.CoreMapActivity.notifShowed;
import static com.juablaz.myplaces.CoreMapActivity.sharedMarkers;
import static com.juablaz.myplaces.CoreMapActivity.userLogged;
import static com.juablaz.myplaces.CoreMapActivity.visibleMarkers;

public class ConfigurationActivity extends AppCompatActivity {

    private static ArrayList<String> colors = new ArrayList<>();


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_configuration);

        if (colors == null || colors.isEmpty()) {

            colors.add("#0000FF"); // Blue
            colors.add("#008080"); // Teal
            colors.add("#000080"); // Navy
            colors.add("#FF00FF"); // Fuchsia
            colors.add("#800080"); // Purple

            colors.add("#FF0000"); // Red
            colors.add("#800000"); // Maroon
            colors.add("#ff6600"); // Orange


            colors.add("#00cc00"); // Green
            colors.add("#008000"); // Green
            colors.add("#808080"); // Gray
            colors.add("#000000"); // Black


        }

        SeekBar seekBar = findViewById(R.id.radius_bar);
        final TextView seekBarValue = findViewById(R.id.seekBarValue);


        SharedPreferences sp12 = getSharedPreferences("radius", MODE_PRIVATE);
        int radius = sp12.getInt("radius", 0);
        Log.i("RADIUS", radius + "");
        seekBarValue.setText(radius + "");
        seekBar.setProgress(radius);

        Button logout = findViewById(R.id.btn_cerrarSesion);
        final Button colorPickerButton = findViewById(R.id.color_picker);

        SharedPreferences sp1 = getSharedPreferences("color", MODE_PRIVATE);
        int color = sp1.getInt("color", 0);

        if (color != 0) {
            colorPickerButton.setBackgroundColor(color);
        }else{
            colorPickerButton.setBackgroundColor(Color.parseColor(colors.get(0)));
        }

        colorPickerButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final ColorPicker colorPicker = new ColorPicker(ConfigurationActivity.this);

                colorPicker.setColors(colors)
                        .setColumns(4)
                        .setRoundColorButton(false)
                        .setOnChooseColorListener(new ColorPicker.OnChooseColorListener() {
                            @Override
                            public void onChooseColor(int position, int color) {
                                if (position < 0 || color == 0) {

                                } else {
                                    colorPickerButton.setBackgroundColor(color);
                                    int colorTransformed = Color.parseColor(colors.get(position));

                                    int r = Color.red(colorTransformed);
                                    int g = Color.green(colorTransformed);
                                    int b = Color.blue(colorTransformed);


                                    SharedPreferences sp = getSharedPreferences("color", MODE_PRIVATE);
                                    SharedPreferences.Editor Ed = sp.edit();
                                    Ed.putInt("r", r);
                                    Ed.putInt("g", g);
                                    Ed.putInt("b", b);
                                    Ed.putInt("color", colorTransformed);

                                    Log.i("COLOR", r + " " + g + " " + b + " " + colorTransformed + "");

                                    colorChanged = true;
                                    Ed.commit();


                                }
                                Log.i("COLOR", color + " " + position);
                                // put code
                            }

                            @Override
                            public void onCancel() {
                                // put code
                            }
                        }).show();

            }
        });


        final Switch notificationSwitch = findViewById(R.id.switch_notification_button);

        //Switch
        SharedPreferences sp = getSharedPreferences("notification", MODE_PRIVATE);
        boolean enabled = sp.getBoolean("notification", true);

        if (enabled) {
            notificationSwitch.setChecked(true);
        } else {
            notificationSwitch.setChecked(false);
        }

        notificationSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                SharedPreferences sp = getSharedPreferences("notification", MODE_PRIVATE);
                SharedPreferences.Editor Ed = sp.edit();
                Ed.putBoolean("notification", notificationSwitch.isChecked());
                Ed.commit();

                if (notificationSwitch.isChecked() == true) {
                    Toast.makeText(ConfigurationActivity.this, "Notificaciones activadas.", Toast.LENGTH_SHORT).show();
                } else {
                    Toast.makeText(ConfigurationActivity.this, "Notificaciones desactivadas.", Toast.LENGTH_SHORT).show();

                }
            }
        });

        logout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                AlertDialog.Builder builder = new AlertDialog.Builder(ConfigurationActivity.this);
                builder.setPositiveButton("Yes", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        new Thread(new Runnable() {

                            @Override
                            public void run() {
                                try {
                                    Log.d("MyFirebaseMsgService", "intentandolo");

                                    FirebaseInstanceId.getInstance().deleteInstanceId();
                                    Log.d("MyFirebaseMsgService", "conseguido");

                                } catch (IOException e) {
                                    Log.d("MyFirebaseMsgService", "error");
                                    e.printStackTrace();
                                }
                            }
                        }).start();
                        setDefaultPreferences();

                        SharedPreferences sp = getSharedPreferences("login", MODE_PRIVATE);
                        SharedPreferences.Editor Ed = sp.edit();
                        Ed.putString("username", "");
                        Ed.putString("password", "");
                        Ed.commit();

                        SharedPreferences sp1 = getSharedPreferences("color", MODE_PRIVATE);
                        SharedPreferences.Editor Ed1 = sp1.edit();
                        Ed1.putInt("r", 0);
                        Ed1.putInt("g", 0);
                        Ed1.putInt("b", 0);
                        Ed1.putInt("color", 0);
                        Ed1.commit();


                        SharedPreferences sp2 = getSharedPreferences("radius", MODE_PRIVATE);
                        SharedPreferences.Editor Ed2 = sp2.edit();
                        Ed2.putInt("radius", 0);
                        Ed2.commit();

                        Intent intentRegister = new Intent(ConfigurationActivity.this, LoginActivity.class);
                        ConfigurationActivity.this.startActivity(intentRegister);
                    }
                });
                builder.setNegativeButton("No", new DialogInterface.OnClickListener()
                {
                    @Override
                    public void onClick(DialogInterface dialog, int which)
                    {
                        // Stuff to do
                        Log.i("MSG",""+which);
                    }
                });

                builder.setMessage("¿Está seguro que desea cerrar sesión?");
                builder.setTitle("Atención");

                AlertDialog d = builder.create();
                d.show();




            }
        });


        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {

            @Override
            public void onProgressChanged(SeekBar seekBar, int progresValue, boolean fromUser) {
                int valorReal = progresValue + 100;
                seekBarValue.setText(String.valueOf(valorReal));
                SharedPreferences sp = getSharedPreferences("radius", MODE_PRIVATE);
                SharedPreferences.Editor Ed = sp.edit();
                Ed.putInt("radius", valorReal);
                Ed.commit();
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {
            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
            }
        });
    }

    private void setDefaultPreferences() {
        visibleMarkers.clear();
        sharedMarkers.clear();
        notifShowed = false;
        userLogged = null;
    }


}
