package com.juablaz.myplaces;

import android.Manifest;
import android.app.Activity;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.NotificationCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonArrayRequest;
import com.android.volley.toolbox.Volley;
import com.example.juancarlos.myplaces.R;
import com.github.clans.fab.FloatingActionButton;
import com.github.clans.fab.FloatingActionMenu;
import com.google.android.gms.common.GooglePlayServicesNotAvailableException;
import com.google.android.gms.common.GooglePlayServicesRepairableException;
import com.google.android.gms.location.places.Place;
import com.google.android.gms.location.places.ui.PlacePicker;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.Circle;
import com.google.android.gms.maps.model.CircleOptions;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.juablaz.myplaces.Util.CustomHurlStack;
import com.juablaz.myplaces.Util.CustomInfoWindowAdapter;
import com.juablaz.myplaces.Util.CustomMarkerComparator;
import com.juablaz.myplaces.Util.CustomResponse;
import com.juablaz.myplaces.Util.DateUtils;
import com.juablaz.myplaces.model.CustomMarker;
import com.juablaz.myplaces.model.SharedMarker;
import com.juablaz.myplaces.model.Usuario;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import static com.juablaz.myplaces.ShowMySharedPlacesActivity.currentSelectedItems;
import static com.juablaz.myplaces.Util.Constants.MAPVIEW_BUNDLE_KEY;
import static com.juablaz.myplaces.Util.StringUtils.ADDED_OK;
import static com.juablaz.myplaces.Util.StringUtils.BAR;
import static com.juablaz.myplaces.Util.StringUtils.CALL_GETALL;
import static com.juablaz.myplaces.Util.StringUtils.CALL_GETSHARED;
import static com.juablaz.myplaces.Util.StringUtils.DELETED_OK;
import static com.juablaz.myplaces.Util.StringUtils.DOUBLE_BAR;
import static com.juablaz.myplaces.Util.StringUtils.HOST;
import static com.juablaz.myplaces.Util.StringUtils.PORT;
import static com.juablaz.myplaces.Util.StringUtils.PROTOCOL;
import static com.juablaz.myplaces.Util.StringUtils.RESPONSE_OK;
import static com.juablaz.myplaces.Util.StringUtils.STATE_2xx;
import static com.juablaz.myplaces.Util.StringUtils.TWO_POINTS;
import static com.juablaz.myplaces.Util.StringUtils.UPDATED_OK;

public class CoreMapActivity extends AppCompatActivity implements Serializable, OnMapReadyCallback, Response.Listener<JSONArray>, Response.ErrorListener {

    private MapView mMapView;
    private static final String TAG = "CoreMapActivity";
    private static final float DEFAULT_ZOOM = 15f;
    private static final int EDIT_REQUEST = 1;
    private static final int PLACE_PICKER_REQUEST = 2;
    private static final int SHARING_PLACES_REQUEST = 3;

    public static int DELAY = 425;
    public static Usuario userLogged;

    private GoogleMap mMap;

    private EditText mSearchText;
    private ImageButton searchButton;
    private FloatingActionMenu fam;

    private Boolean adding = false;
    public static Boolean notifShowed = false;
    private Boolean addingClosePlace = false;
    private Boolean removing = false;
    private Boolean editing = false;
    private Boolean isInfoWindowShown = false;
    boolean fromNotifToCore = false;

    public static ArrayList<CustomMarker> visibleMarkers = new ArrayList<>();
    public static ArrayList<SharedMarker> sharedMarkers = new ArrayList<>();
    private ArrayList<CustomMarker> nowAddedMarkers = new ArrayList<>();
    private ArrayList<CustomMarker> nowRemovedMarkers = new ArrayList<>();
    private ArrayList<MarkerOptions> nowEditedMarkers = new ArrayList<>();
    private ArrayList<CustomMarker> newEditedMarkers = new ArrayList<>();

    public static Location currentLocation;
    private Circle circle;


    private Marker markerClicked;

    public static boolean colorChanged = false;
    private static int strokeColor;
    private static int fillColor;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_core_map);

        FloatingActionButton fabAddPlace = findViewById(R.id.fab1);
        FloatingActionButton fabAddClosePlace = findViewById(R.id.fab2);
        FloatingActionButton fabEditPlace = findViewById(R.id.fab3);
        FloatingActionButton fabRemovePlace = findViewById(R.id.fab4);
        FloatingActionButton fabShowMyPlaces = findViewById(R.id.fab5);
        FloatingActionButton fabShowMySharedPlaces = findViewById(R.id.fab6);
        FloatingActionButton fabConfiguration = findViewById(R.id.fab7);

        SharedPreferences sp12 = getSharedPreferences("radius", MODE_PRIVATE);
        int radius = sp12.getInt("radius", 0);
        if (radius <= 0) {
            radius = 300;
            SharedPreferences sp = getSharedPreferences("radius", MODE_PRIVATE);
            SharedPreferences.Editor Ed = sp.edit();
            Ed.putInt("radius", radius);
            Ed.commit();
        }


        fam = findViewById(R.id.menu);

        fam.setOnMenuButtonClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Log.i(TAG, "Visible markers size: " + visibleMarkers.size());
                if (nowAddedMarkers.isEmpty() && addingClosePlace) {
                    addingClosePlace = false;
                }

                if (adding || addingClosePlace) {
                    Log.i(TAG, "Now Added markers size: " + nowAddedMarkers.size());

                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    adding = false;
                    addingClosePlace = false;

                    //Añado esos lugares al total de lugares.
                    for (CustomMarker customMarker : nowAddedMarkers) {
                        for (CustomMarker customMarker1 : visibleMarkers) {
                            if (customMarker.getMarker().getPosition().equals(customMarker1.getMarker().getPosition())) {
                                nowAddedMarkers.remove(customMarker);
                                Toast.makeText(CoreMapActivity.this, "Ese lugar ya existe, elige otro.", Toast.LENGTH_SHORT).show();
                            }
                        }
                    }
                    visibleMarkers.addAll(nowAddedMarkers);

                    if (!nowAddedMarkers.isEmpty()) {
                        callWebService("add");
                    }
                    nowAddedMarkers.clear();


                } else if (removing) {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    removing = false;

                    for (CustomMarker customMarker : nowRemovedMarkers) {
                        Marker marker = customMarker.getMarker();
                        Log.i(TAG, "Marker Borrado: " + marker.getTitle() + " " + marker.getSnippet());
                        marker.remove();
                    }
                    //Elimino esos lugares del total de lugares.
                    visibleMarkers.removeAll(nowRemovedMarkers);

                    if (!nowRemovedMarkers.isEmpty()) {
                        callWebService("remove");
                    }
                    nowRemovedMarkers.clear();
                } else if (editing) {
                    if (isInfoWindowShown) {

                        CustomInfoWindowAdapter customInfoWindowAdapter = new CustomInfoWindowAdapter(CoreMapActivity.this);
                        customInfoWindowAdapter.getInfoWindow(markerClicked).findViewById(R.id.imageView).setVisibility(View.INVISIBLE);
                        customInfoWindowAdapter.getInfoWindow(markerClicked).findViewById(R.id.spacesView).setVisibility(View.INVISIBLE);
                        mMap.setInfoWindowAdapter(customInfoWindowAdapter);

                        markerClicked.showInfoWindow();


                    }

                    Log.i(TAG, "Editando");

                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    editing = false;

                    //Elimino esos lugares del total de lugares.
                    if (!newEditedMarkers.isEmpty()) {
                        callWebService("edit");
                    }

                    visibleMarkers.removeAll(newEditedMarkers);
                    visibleMarkers.addAll(newEditedMarkers);

                    nowEditedMarkers.clear();
                    newEditedMarkers.clear();

                } else {
                    Log.i(TAG, "Esta abierto?: " + fam.isOpened());

                    if (fam.isOpened()) {
                        fam.close(true);
                    } else {
                        fam.open(true);
                    }
                }


                Log.i(TAG, "Markers totales: " + visibleMarkers.size());

            }
        });

        fabAddPlace.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!adding) {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.ic_save);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    adding = true;
                    removing = false;
                    fam.close(true);
                    Toast.makeText(CoreMapActivity.this, "Haga click en el mapa para añadir lugares", Toast.LENGTH_SHORT).show();

                } else {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    fam.close(true);
                    adding = false;
                }
            }
        });

        fabRemovePlace.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!removing) {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.ic_save);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    removing = true;
                    adding = false;
                    fam.close(true);
                    Toast.makeText(CoreMapActivity.this, "Haga click en los lugares para borrarlos", Toast.LENGTH_SHORT).show();

                } else {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    fam.close(true);
                    removing = false;
                }

            }
        });

        fabEditPlace.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                loadCustomInfoWindowEdit();

                if (!editing) {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.ic_save);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    editing = true;
                    adding = false;
                    removing = false;
                    fam.close(true);
                    Toast.makeText(CoreMapActivity.this, "Haga click en los lugares para editarlos", Toast.LENGTH_SHORT).show();
                } else {
                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
                    fam.getMenuIconView().setImageDrawable(drawable);
                    fam.close(true);
                    editing = false;
                }

            }
        });

        fabAddClosePlace.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                addingClosePlace = true;
                PlacePicker.IntentBuilder builder = new PlacePicker.IntentBuilder();
                fam.close(true);

                try {
                    startActivityForResult(builder.build(CoreMapActivity.this), PLACE_PICKER_REQUEST);
                } catch (GooglePlayServicesRepairableException | GooglePlayServicesNotAvailableException e) {
                    e.printStackTrace();
                }

            }
        });

        fabShowMyPlaces.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fam.close(true);

                final Intent edit = new Intent(CoreMapActivity.this, ShowPlacesActivity.class);

                for (CustomMarker customMarker : visibleMarkers) {
                    Location markerLocation = new Location("");
                    markerLocation.setLatitude(customMarker.getMarker().getPosition().latitude);
                    markerLocation.setLongitude(customMarker.getMarker().getPosition().longitude);

                    customMarker.setDistancia(currentLocation.distanceTo(markerLocation));


                }
                Collections.sort(visibleMarkers, new CustomMarkerComparator());

                Handler handler = new Handler();

                handler.postDelayed(new Runnable() {
                    public void run() {
                        CoreMapActivity.this.startActivityForResult(edit, EDIT_REQUEST);
                    }
                }, DELAY);

            }
        });

        fabShowMySharedPlaces.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fam.close(true);

                final Intent showSharedPlaces = new Intent(CoreMapActivity.this, ShowMySharedPlacesActivity.class);
                callWebService("getSharedPlaces");
                Log.i(TAG, "Shared markers size: " + sharedMarkers.size());

                Handler handler = new Handler();

                handler.postDelayed(new Runnable() {
                    public void run() {
                        startActivityForResult(showSharedPlaces, SHARING_PLACES_REQUEST);

                    }
                }, DELAY);

            }
        });

        fabConfiguration.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fam.close(true);

                final Intent edit = new Intent(CoreMapActivity.this, ConfigurationActivity.class);

                Handler handler = new Handler();

                handler.postDelayed(new Runnable() {
                    public void run() {
                        CoreMapActivity.this.startActivity(edit);
                    }
                }, DELAY);

            }
        });

        Bundle mapViewBundle = null;
        if (savedInstanceState != null) {
            mapViewBundle = savedInstanceState.getBundle(MAPVIEW_BUNDLE_KEY);
        }

        mMapView = findViewById(R.id.map);
        mMapView.onCreate(mapViewBundle);
        mMapView.getMapAsync(this);

        mSearchText = findViewById(R.id.input_user);
        searchButton = findViewById(R.id.search_button);

        if (userLogged == null) {
            userLogged = (Usuario) getIntent().getSerializableExtra("user");
        }


    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);

        Bundle mapViewBundle = outState.getBundle(MAPVIEW_BUNDLE_KEY);
        if (mapViewBundle == null) {
            mapViewBundle = new Bundle();
            outState.putBundle(MAPVIEW_BUNDLE_KEY, mapViewBundle);
        }

        mMapView.onSaveInstanceState(mapViewBundle);
    }

    @Override
    protected void onResume() {
        super.onResume();
        mMapView.onResume();
    }

    @Override
    protected void onStart() {
        super.onStart();
        mMapView.onStart();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mMapView.onStop();
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // TODO: Remove Consider calling
            //    ActivityCompat#requestPermissions
            // here to request the missing permissions, and then overriding
            //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
            //                                          int[] grantResults)
            // to handle the case where the user grants the permission. See the documentation
            // for ActivityCompat#requestPermissions for more details.
            return;
        }

        mMap = googleMap;
        mMap.setMyLocationEnabled(true);
        mMap.getUiSettings().setMapToolbarEnabled(false); //No mostrar las opciones de navegación hasta el lugar porque se superponen con mi boton flotante.
        mMap.setInfoWindowAdapter(new CustomInfoWindowAdapter(CoreMapActivity.this));
        mMap.setBuildingsEnabled(true);
        mMap.getUiSettings().setRotateGesturesEnabled(true);

        if (visibleMarkers.isEmpty()) {
            callWebService("getAll");
        } else {
            for (CustomMarker cm : visibleMarkers) {
                MarkerOptions options = new MarkerOptions()
                        .snippet(cm.getMarker().getSnippet())
                        .position(cm.getMarker().getPosition())
                        .draggable(true)
                        .title(cm.getMarker().getTitle());

                mMap.addMarker(options);
            }
        }
        fromNotifToCore = getIntent().getBooleanExtra("fromNotifToCore", false);

        if (fromNotifToCore) {
            for (CustomMarker customMarker : visibleMarkers) {
                customMarker.getMarker().remove();
                mMap.clear();
            }
            sharedMarkers.removeAll(currentSelectedItems);
            visibleMarkers.clear();


            callWebService("getAll");

        }
        if (sharedMarkers.isEmpty()) {
            callWebService("getSharedPlaces");
        }


        mMap.setOnMapClickListener(new GoogleMap.OnMapClickListener() {
            @Override
            public void onMapClick(LatLng point) {
                if (adding) {
                    Geocoder geocoder = new Geocoder(CoreMapActivity.this);
                    List<Address> list = new ArrayList<>();
                    Double latitude = point.latitude;
                    Double longitude = point.longitude;
                    LatLng latLng = new LatLng(latitude, longitude);
                    try {

                        list = geocoder.getFromLocation(latitude, longitude, 1);
                    } catch (IOException e) {
                        Log.e(TAG, "geoLocate: IOException: " + e.getMessage());
                    }

                    if (list.size() > 0) {
                        Address address = list.get(0);

                        MarkerOptions options = new MarkerOptions()
                                .snippet("Dirección: " + address.getAddressLine(0))
                                .position(latLng)
                                .draggable(true);
                        if (address.getThoroughfare() != null) {
                            options.title(address.getThoroughfare());
                        } else {
                            options.title(address.getFeatureName());
                        }

                        Marker marker = mMap.addMarker(options);

                        CustomMarker customMarker = new CustomMarker(marker);
                        nowAddedMarkers.add(customMarker);

                        mMap.animateCamera(CameraUpdateFactory.newLatLng(latLng));

                    }

                }

                isInfoWindowShown = false;
                System.out.println(point.latitude + "---" + point.longitude);
            }
        });

        mMap.setOnInfoWindowClickListener(new GoogleMap.OnInfoWindowClickListener() {
            @Override
            public void onInfoWindowClick(Marker marker) {
                if (editing) {
                    Intent edit = new Intent(CoreMapActivity.this, EditPlaceActivity.class);
                    edit.putExtra("location", marker.getPosition());
                    edit.putExtra("title", marker.getTitle());
                    edit.putExtra("direction", marker.getSnippet());
                    edit.putExtra("id", marker.getId());
                    edit.putExtra("latitude", marker.getPosition().latitude);
                    edit.putExtra("longitude", marker.getPosition().longitude);

                    Log.i("latlon", marker.getPosition().latitude + ",xD ,  " + marker.getPosition().longitude);


                    marker.hideInfoWindow();
                    isInfoWindowShown = false;
                    CoreMapActivity.this.startActivityForResult(edit, EDIT_REQUEST);
                }
                if (!editing && !adding && !removing) {
                    Intent edit = new Intent(CoreMapActivity.this, ViewPlaceActivity.class);
                    edit.putExtra("currentLocationLatitude", currentLocation.getLatitude());
                    edit.putExtra("currentLocationLongitude", currentLocation.getLongitude());
                    edit.putExtra("title", marker.getTitle());
                    edit.putExtra("snippet", marker.getSnippet());
                    edit.putExtra("isNotification", false);

                    Log.i(TAG, marker.getTitle() + marker.getSnippet());

                    marker.hideInfoWindow();
                    isInfoWindowShown = false;
                    CoreMapActivity.this.startActivityForResult(edit, EDIT_REQUEST);

                }
            }
        });


        mMap.setOnMarkerClickListener(new GoogleMap.OnMarkerClickListener() {
            @Override
            public boolean onMarkerClick(Marker marker) {
                markerClicked = marker;
                if (editing) {
                    CustomInfoWindowAdapter customInfoWindowAdapter = new CustomInfoWindowAdapter(CoreMapActivity.this);
                    mMap.setInfoWindowAdapter(customInfoWindowAdapter);
                } else {
                    CustomInfoWindowAdapter customInfoWindowAdapter = new CustomInfoWindowAdapter(CoreMapActivity.this);
                    customInfoWindowAdapter.getInfoWindow(marker).findViewById(R.id.imageView).setVisibility(View.INVISIBLE);
                    customInfoWindowAdapter.getInfoWindow(marker).findViewById(R.id.spacesView).setVisibility(View.INVISIBLE);
                    mMap.setInfoWindowAdapter(customInfoWindowAdapter);
                }

                if (removing) {
                    mMap.animateCamera(CameraUpdateFactory.newLatLng(marker.getPosition()));
                    CustomMarker customMarker = new CustomMarker(marker);

                    nowRemovedMarkers.add(customMarker);
                    marker.setVisible(false);

                } else {
                    if (!isInfoWindowShown) {
                        marker.showInfoWindow();
                        mMap.animateCamera(CameraUpdateFactory.newLatLng(marker.getPosition()));
                        isInfoWindowShown = true;
                    } else {
                        marker.hideInfoWindow();
                        isInfoWindowShown = false;
                    }

                    return true;
                }
                return true;
            }
        });

        searchButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (!mSearchText.getText().toString().isEmpty()) {
                    geoLocate();
                    hideSoftKeyboard();
                } else {
                    Toast.makeText(CoreMapActivity.this, "Debe introducir una dirección", Toast.LENGTH_SHORT).show();

                }
            }
        });

        LocationListener mLocationListener = new LocationListener() {
            @Override
            public void onLocationChanged(Location location) {
/*
                Log.i("CIRCLE", "location: " + location.toString() + "!");
*/

                currentLocation = location;
                if (circle != null) {
                    circle.remove();
                }

                //Default
                SharedPreferences sp = getSharedPreferences("color", MODE_PRIVATE);
                int r = sp.getInt("r", 0);
                int g = sp.getInt("g", 0);
                int b = sp.getInt("b", 0);
                int color = sp.getInt("color", 0);

                if (r == 0 && g == 0 && b == 0 && color == 0) {
                    strokeColor = -16776961;
                    fillColor = 1342177535;
                } else {
                    fillColor = Color.argb(80, r, g, b);
                    strokeColor = color;
                }


                CircleOptions circleOptions = new CircleOptions()
                        .radius(getRadiusValue())
                        .strokeColor(strokeColor)
                        .center(new LatLng(location.getLatitude(), location.getLongitude()))
                        .strokeWidth(3f)
                        .fillColor(fillColor);

                circle = mMap.addCircle(circleOptions);



                for (CustomMarker customMarker : visibleMarkers) {
                    Marker marker = customMarker.getMarker();
                    Location markerLocation = new Location("");
                    markerLocation.setLatitude(marker.getPosition().latitude);
                    markerLocation.setLongitude(marker.getPosition().longitude);


                    if (location.distanceTo(markerLocation) < getRadiusValue()) {
                        if (!customMarker.isSentNotification()) {
                            addNotification(marker);
                            customMarker.setSentNotification(true);
/*
                            Log.i(TAG, "Distancia al marker: " + location.distanceTo(markerLocation));
*/
                        }

                    } else {
                        customMarker.setSentNotification(false);
                    }

                }

            }

            @Override
            public void onStatusChanged(String provider, int status, Bundle extras) {

            }

            @Override
            public void onProviderEnabled(String provider) {

            }

            @Override
            public void onProviderDisabled(String provider) {

            }
        };

        LocationManager locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // TODO: Consider calling
            //    ActivityCompat#requestPermissions
            // here to request the missing permissions, and then overriding
            //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
            //                                          int[] grantResults)
            // to handle the case where the user grants the permission. See the documentation
            // for ActivityCompat#requestPermissions for more details.
            return;
        }
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 5 * 1000 / 10,
                0, mLocationListener);

        if (currentLocation != null) {
            LatLng latLng = new LatLng(currentLocation.getLatitude(), currentLocation.getLongitude());
            mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, CoreMapActivity.DEFAULT_ZOOM));
        }

    }


    private void geoLocate() {
        Log.d(TAG, "geoLocate: geolocating");

        String searchString = mSearchText.getText().toString();


        Geocoder geocoder = new Geocoder(CoreMapActivity.this);
        List<Address> list = new ArrayList<>();
        try {
            list = geocoder.getFromLocationName(searchString, 1);
        } catch (IOException e) {
            Log.e(TAG, "geoLocate: IOException: " + e.getMessage());
        }

        if (list.size() > 0) {
            Address address = list.get(0);
            Log.d(TAG, "geoLocate: found a location: " + address.toString());
            moveCamera(new LatLng(address.getLatitude(), address.getLongitude()));
        } else {
            Toast.makeText(CoreMapActivity.this, "No se ha encontrado un lugar con esa dirección", Toast.LENGTH_SHORT).show();
        }
    }

    private void moveCamera(LatLng latLng) {
        Log.d(TAG, "moveCamera: moving the camera to: lat: " + latLng.latitude + ", lng: " + latLng.longitude);
        mMap.animateCamera(CameraUpdateFactory.newLatLngZoom(latLng, CoreMapActivity.DEFAULT_ZOOM));

        hideSoftKeyboard();
    }


    private void hideSoftKeyboard() {
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(mSearchText.getWindowToken(), 0);
    }


    @Override
    protected void onPause() {
        mMapView.onPause();
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        mMapView.onDestroy();
        super.onDestroy();
    }

    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mMapView.onLowMemory();
    }


    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            case (EDIT_REQUEST): {
                if (resultCode == Activity.RESULT_OK) {
                    String title = data.getStringExtra("title");
                    String snippet = data.getStringExtra("snippet");
                    String id = data.getStringExtra("id");
                    double lat = data.getDoubleExtra("idLat", 0);
                    double lon = data.getDoubleExtra("idLong", 0);

                    LatLng latLng = new LatLng(lat, lon);

                    Log.i("latlon", lat + ", " + lon);
                    if (findMarkerByPosition(latLng) == null) {
                        Log.i("latlon", "ES NULO");

                    }
                    Log.i("latlon", findMarkerByPosition(latLng).toString());


                    MarkerOptions options = new MarkerOptions()
                            .snippet(findMarkerByPosition(latLng).getSnippet())
                            .position(findMarkerByPosition(latLng).getPosition())
                            .draggable(true)
                            .title(findMarkerByPosition(latLng).getTitle());


                    nowEditedMarkers.add(options);

                    Marker newEditedMarker = findMarkerByPosition(latLng);

                    newEditedMarker.setTitle(title);
                    newEditedMarker.setSnippet(snippet);

                    boolean added = false;
                    for (CustomMarker myMarker : newEditedMarkers) {
                        if (myMarker.getMarker().getPosition().equals(newEditedMarker.getPosition())) {
                            myMarker.getMarker().setTitle(newEditedMarker.getTitle());
                            myMarker.getMarker().setSnippet(newEditedMarker.getSnippet());
                            added = true;
                        }
                    }
                    if (!added) {
                        newEditedMarkers.add(new CustomMarker(newEditedMarker));

                    }
                    if (adding) {
                        Marker nowEditedMarker = findMarkerByPosition(latLng);
                        nowEditedMarker.setTitle(newEditedMarker.getTitle());
                        nowEditedMarker.setSnippet(newEditedMarker.getSnippet());
                        nowEditedMarker.showInfoWindow();

                        isInfoWindowShown = true;
                    }

                }
                break;

            }
            case (PLACE_PICKER_REQUEST): {
                if (resultCode == RESULT_OK) {
                    addingClosePlace = true;

                    Place place = PlacePicker.getPlace(this, data);

                    MarkerOptions options = new MarkerOptions()
                            .snippet("Dirección: " + place.getAddress())
                            .position(place.getLatLng())
                            .draggable(true)
                            .title(place.getName().toString());

                    Marker marker = mMap.addMarker(options);
                    CustomMarker customMarker = new CustomMarker(marker);

                    nowAddedMarkers.add(customMarker);
                    mMap.animateCamera(CameraUpdateFactory.newLatLng(place.getLatLng()));

                    Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.ic_save);
                    fam.getMenuIconView().setImageDrawable(drawable);

                }
                break;

            }
            case (SHARING_PLACES_REQUEST): {
                if (resultCode == RESULT_OK) {
                    for (SharedMarker sharedMarker : currentSelectedItems) {

                        for (CustomMarker customMarker : visibleMarkers) {
                            if (customMarker.getMarker().getPosition().equals(sharedMarker.getMarkerOptions().getPosition())) {
                                visibleMarkers.remove(customMarker);
                                Marker marker = customMarker.getMarker();
                                marker.remove();

                            }
                        }

                        MarkerOptions markerOptions = sharedMarker.getMarkerOptions();

                        Marker marker = mMap.addMarker(markerOptions);

                        visibleMarkers.add(new CustomMarker(marker));


                    }
                    sharedMarkers.removeAll(currentSelectedItems);
                }
                break;

            }

        }
    }

    public Marker findMarkerByPosition(LatLng position) {
        if (adding) {
            for (CustomMarker customMarker : nowAddedMarkers) {
                Marker marker = customMarker.getMarker();
                if (marker.getPosition().latitude == position.latitude && marker.getPosition().longitude == position.longitude) {
                    return marker;
                }
            }

        }

        if (removing) {
            for (CustomMarker customMarker : nowRemovedMarkers) {
                Marker marker = customMarker.getMarker();
                if (marker.getPosition().latitude == position.latitude && marker.getPosition().longitude == position.longitude) {
                    return marker;
                }
            }

        } else {
            for (CustomMarker customMarker : visibleMarkers) {
                Marker marker = customMarker.getMarker();
                Log.i("latlon", "" + marker.getPosition().latitude);
                Log.i("latlon", "" + marker.getPosition().longitude);
                Log.i("latlon", "mandado " + position.latitude);
                Log.i("latlon", "mandado " + position.longitude);

                if (marker.getPosition().latitude == position.latitude && marker.getPosition().longitude == position.longitude) {
                    return marker;
                }
            }
        }

        return null;
    }

    @Override
    public void onBackPressed() {
        if (removing) {
            if (!nowRemovedMarkers.isEmpty()) {
                for (CustomMarker customMarker : nowRemovedMarkers) {
                    Marker marker = customMarker.getMarker();
                    marker.setVisible(true);
                }
                nowRemovedMarkers.clear();
                Toast.makeText(CoreMapActivity.this, "Lugares borrados descartados", Toast.LENGTH_SHORT).show();
            }
            Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
            fam.getMenuIconView().setImageDrawable(drawable);
            removing = false;


        } else if (adding || addingClosePlace) {
            if (!nowAddedMarkers.isEmpty()) {
                for (CustomMarker customMarker : nowAddedMarkers) {
                    Marker marker = customMarker.getMarker();
                    marker.remove();
                }
                nowAddedMarkers.clear();
                Toast.makeText(CoreMapActivity.this, "Lugares añadidos descartados", Toast.LENGTH_SHORT).show();
            }
            Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
            fam.getMenuIconView().setImageDrawable(drawable);
            adding = false;

        } else if (editing) {
            if (markerClicked != null) {
                markerClicked.hideInfoWindow();
            }
            if (!newEditedMarkers.isEmpty()) {
                for (int i = 0; i < newEditedMarkers.size(); i++) {
                    CustomMarker customMarker = newEditedMarkers.get(i);
                    MarkerOptions markerOptions = nowEditedMarkers.get(i);

                    Marker marker = customMarker.getMarker();


                    marker.setTitle(markerOptions.getTitle());
                    marker.setSnippet(markerOptions.getSnippet());
                }
                newEditedMarkers.clear();
                nowEditedMarkers.clear();


                Toast.makeText(CoreMapActivity.this, "Lugares editados descartados", Toast.LENGTH_SHORT).show();
            }
            Drawable drawable = ContextCompat.getDrawable(getApplicationContext(), R.drawable.fab_add);
            fam.getMenuIconView().setImageDrawable(drawable);
            editing = false;


        }
    }

    // Call My Web Service
    private void callWebService(String operation) {
        JSONArray jsonBody = new JSONArray();

        CustomHurlStack customHurlStack = new CustomHurlStack();
        RequestQueue requestQueue = Volley.newRequestQueue(this, customHurlStack);

        String url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/places";


        try {

            JSONObject userJSON = new JSONObject();
            userJSON.put("username", userLogged.getNombreUsuario());
            jsonBody.put(userJSON);

            JSONArray markers = new JSONArray();
            Log.i("###########", "###### SELECCIONANDO OPERACION #####");

            switch (operation) {
                case "add":
                    Log.i("###########", "###### ADD #####");

                    for (CustomMarker customMarker : nowAddedMarkers) {
                        Marker marker = customMarker.getMarker();

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("title", marker.getTitle());
                        markerJSON.put("snippet", marker.getSnippet());
                        markerJSON.put("latitude", marker.getPosition().latitude);
                        markerJSON.put("longitude", marker.getPosition().longitude);
                        markers.put(markerJSON);

                    }
                    jsonBody.put(markers);
                    break;


                case "remove":
                    Log.i("###########", "###### REMOVE #####");

                    //LLamada para borrar
                    for (CustomMarker customMarker : nowRemovedMarkers) {
                        Marker marker = customMarker.getMarker();

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("title", marker.getTitle());
                        markerJSON.put("snippet", marker.getSnippet());
                        markerJSON.put("latitude", marker.getPosition().latitude);
                        markerJSON.put("longitude", marker.getPosition().longitude);
                        markers.put(markerJSON);
                    }
                    jsonBody.put(markers);
                    break;

                case "getAll":
                    Log.i("###########", "###### GETALL #####");
                    break;

                case "getSharedPlaces":
                    Log.i("###########", "###### GETSHAREDPLACES #####");
                    break;

                case "edit":
                    Log.i("###########", "###### EDIT #####");

                    for (CustomMarker customMarker : newEditedMarkers) {
                        Marker marker = customMarker.getMarker();

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("title", marker.getTitle());
                        markerJSON.put("snippet", marker.getSnippet());
                        markerJSON.put("latitude", marker.getPosition().latitude);
                        markerJSON.put("longitude", marker.getPosition().longitude);
                        markers.put(markerJSON);
                    }
                    jsonBody.put(markers);

                    break;
            }

            Log.i("###########", jsonBody.toString());

        } catch (JSONException e) {
            e.printStackTrace();
        }

        switch (operation) {
            case "add":
                JsonArrayRequest jsonArrayAddRequest = new JsonArrayRequest(Request.Method.POST, url, jsonBody, this, this);
                requestQueue.add(jsonArrayAddRequest);
                break;

            case "remove":
                JsonArrayRequest jsonArrayRemoveRequest = new JsonArrayRequest(Request.Method.DELETE, url, jsonBody, this, this);
                requestQueue.add(jsonArrayRemoveRequest);
                break;

            case "getAll":
                String getUrl = String.format(url + "?username=%1$s", userLogged.getNombreUsuario());

                JsonArrayRequest jsonArrayGetAllRequest = new JsonArrayRequest(getUrl, this, this);
                requestQueue.add(jsonArrayGetAllRequest);
                break;

            case "getSharedPlaces":
                url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/shared_places";
                String sharedUrl = String.format(url + "?username=%1$s", userLogged.getNombreUsuario());
                JsonArrayRequest jsonArrayGetSharedPlacesRequest = new JsonArrayRequest(sharedUrl, this, this);
                requestQueue.add(jsonArrayGetSharedPlacesRequest);
                break;

            case "edit":
                String editUrl = url + "/place";
                JsonArrayRequest jsonArrayEditRequest = new JsonArrayRequest(Request.Method.PUT, editUrl, jsonBody, this, this);
                requestQueue.add(jsonArrayEditRequest);

        }
    }


    @Override
    public void onErrorResponse(VolleyError error) {

        if (error != null) {
            Log.i("ERROR encontrado: ", error.toString());
            Toast.makeText(this, "No se ha podido conectaro con el servidor. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
            Log.i("ERROR", error.toString());
        }
    }


    @Override
    public void onResponse(JSONArray response) {
        if (response != null)
            Log.i("La respuesta es: ", response.toString());

        CustomResponse customResponse;
        if (response != null) {
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject = response.getJSONObject(0);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            customResponse = new CustomResponse(jsonObject.optString("status"),
                    jsonObject.optString("description"),
                    jsonObject.optString("message"),
                    jsonObject.optString("timeStamp"));
            if (jsonObject.optString("call") != null) {
                customResponse.setCall(jsonObject.optString("call"));
            }
            if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(ADDED_OK)) {
                Toast.makeText(CoreMapActivity.this, "Lugares añadidos correctamente", Toast.LENGTH_SHORT).show();
            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(DELETED_OK)) {
                Toast.makeText(this, "Lugares eliminados correctamente", Toast.LENGTH_LONG).show();
            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(UPDATED_OK)) {
                Toast.makeText(this, "Lugares actualizados correctamente", Toast.LENGTH_LONG).show();
                loadCustomInfoWindowShow();

            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && !customResponse.getMessage().toString().isEmpty() && customResponse.getCall() != null && customResponse.getCall().equals(CALL_GETALL)) {
                JSONArray jsonGetAllArray = null;
                Log.i("####", customResponse.getMessage().toString());
                try {
                    jsonGetAllArray = new JSONArray(customResponse.getMessage().toString());
                } catch (JSONException e) {
                    e.printStackTrace();
                }

                if (jsonGetAllArray != null) {
                    visibleMarkers = new ArrayList<>();
                    visibleMarkers.clear();

                    mMap.clear();
                    for (int i = 0; i < jsonGetAllArray.length(); i++) {
                        try {
                            JSONObject jsonMarkerObject = jsonGetAllArray.getJSONObject(i);
                            LatLng latLng = new LatLng(Double.valueOf(jsonMarkerObject.getString("latitude")),
                                    Double.valueOf(jsonMarkerObject.getString("longitude")));

                            MarkerOptions options = new MarkerOptions()
                                    .snippet(jsonMarkerObject.getString("snippet"))
                                    .position(latLng)
                                    .draggable(true)
                                    .title(jsonMarkerObject.getString("title"));

                            Marker marker = mMap.addMarker(options);

                            visibleMarkers.add(new CustomMarker(marker));

                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }
                    if (!visibleMarkers.isEmpty()) {
                        if (!fromNotifToCore) {

                            Toast.makeText(this, "Lugares cargados correctamente.", Toast.LENGTH_LONG).show();
                        } else {
                            Toast.makeText(this, "Lugares añadidos correctamente.", Toast.LENGTH_LONG).show();
                        }
                    }

                }


            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && !customResponse.getMessage().toString().isEmpty() && customResponse.getCall() != null && customResponse.getCall().equals(CALL_GETSHARED)) {
                JSONArray jsonGetAllArray = null;
                Log.i("####", customResponse.getMessage().toString());
                try {
                    jsonGetAllArray = new JSONArray(customResponse.getMessage().toString());
                } catch (JSONException e) {
                    e.printStackTrace();
                }

                if (jsonGetAllArray != null) {
                    sharedMarkers = new ArrayList<>();
                    for (int i = 0; i < jsonGetAllArray.length(); i++) {
                        try {
                            JSONObject jsonMarkerObject = jsonGetAllArray.getJSONObject(i);
                            LatLng latLng = new LatLng(Double.valueOf(jsonMarkerObject.getString("latitude")),
                                    Double.valueOf(jsonMarkerObject.getString("longitude")));

                            MarkerOptions options = new MarkerOptions()
                                    .snippet(jsonMarkerObject.getString("snippet"))
                                    .position(latLng)
                                    .draggable(true)
                                    .title(jsonMarkerObject.getString("title"));

                            SharedMarker sharedMarker = new SharedMarker();
                            sharedMarker.setMarkerOptions(options);
                            sharedMarker.setSentBy((jsonMarkerObject.getString("username")));


                            sharedMarker.setSent(DateUtils.formatToDateUser(jsonMarkerObject.getString("date")));

                            sharedMarkers.add(sharedMarker);

                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }

                    if (!sharedMarkers.isEmpty() && !notifShowed) {
                        notifShowed = true;
                        addSharedPlacesNotification();
                    }
                    Log.i(TAG, "COMPARTIDOS: " + sharedMarkers.size());

                }

            }


        } else {
            Toast.makeText(this, "No se ha conectar con el servidor. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
        }

    }

    private void loadCustomInfoWindowEdit() {
        if (markerClicked != null && isInfoWindowShown) {

            CustomInfoWindowAdapter customInfoWindowAdapter = new CustomInfoWindowAdapter(CoreMapActivity.this);
            customInfoWindowAdapter.getInfoWindow(markerClicked).findViewById(R.id.imageView).setVisibility(View.VISIBLE);
            customInfoWindowAdapter.getInfoWindow(markerClicked).findViewById(R.id.spacesView).setVisibility(View.VISIBLE);
            mMap.setInfoWindowAdapter(customInfoWindowAdapter);
            markerClicked.showInfoWindow();

        }
    }

    private void loadCustomInfoWindowShow() {
        if (markerClicked != null && isInfoWindowShown) {

            CustomInfoWindowAdapter customInfoWindowAdapter = new CustomInfoWindowAdapter(CoreMapActivity.this);
            customInfoWindowAdapter.getInfoWindow(markerClicked).findViewById(R.id.imageView).setVisibility(View.INVISIBLE);
            customInfoWindowAdapter.getInfoWindow(markerClicked).findViewById(R.id.spacesView).setVisibility(View.INVISIBLE);
            mMap.setInfoWindowAdapter(customInfoWindowAdapter);
            markerClicked.showInfoWindow();

        }
    }

    // Creates and displays a notification
    private void addNotification(Marker marker) {
        SharedPreferences sp = getSharedPreferences("notification", MODE_PRIVATE);
        boolean enabled = sp.getBoolean("notification", true);

        if (enabled) {
            Intent notificationIntent = new Intent(CoreMapActivity.this, ViewPlaceActivity.class);

            notificationIntent.putExtra("currentLocationLatitude", currentLocation.getLatitude());
            notificationIntent.putExtra("currentLocationLongitude", currentLocation.getLongitude());

            notificationIntent.putExtra("title", marker.getTitle());
            notificationIntent.putExtra("snippet", marker.getSnippet());
            notificationIntent.putExtra("isNotification", true);

            notificationIntent.setAction(Intent.ACTION_MAIN);
            notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);

            notificationIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_CLEAR_TASK);
            PendingIntent pIntent = PendingIntent.getActivity(CoreMapActivity.this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);

            // Builds your notification
            NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(CoreMapActivity.this, "default")
                    .setSmallIcon(R.mipmap.ic_launcher_round)
                    .setContentTitle(marker.getTitle())
                    .setContentText("Lugar cercano disponible")
                    .setVibrate(new long[]{1000, 1000, 1000, 1000, 1000})
                    .setContentIntent(pIntent)
                    .setPriority(NotificationCompat.PRIORITY_MAX);


            NotificationManager manager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

            manager.notify(UUID.randomUUID().hashCode(), notificationBuilder.setAutoCancel(true).build());
        }
    }

    // Creates and displays a notification
    private void addSharedPlacesNotification() {
        SharedPreferences sp = getSharedPreferences("notification", MODE_PRIVATE);
        boolean enabled = sp.getBoolean("notification", true);

        if (enabled) {
            Intent notificationIntent = new Intent(this, ShowMySharedPlacesActivity.class);

            notificationIntent.putExtra("isNotification", true);
            notificationIntent.putExtra("fromNotification", true);

            notificationIntent.setAction(Intent.ACTION_MAIN);
            notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);

            notificationIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_CLEAR_TASK);
            PendingIntent pIntent = PendingIntent.getActivity(CoreMapActivity.this, 0, notificationIntent, 0);

            // Builds your notification
            NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(CoreMapActivity.this, "default")
                    .setSmallIcon(R.mipmap.ic_launcher_round)
                    .setContentTitle("Tiene lugares compartidos")
                    .setContentText("Añada o rechace esos lugares")
                    .setVibrate(new long[]{1000, 1000, 1000, 1000, 1000})
                    .setContentIntent(pIntent)
                    .setPriority(NotificationCompat.PRIORITY_MAX);

            NotificationManager manager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

            manager.notify(UUID.randomUUID().hashCode(), notificationBuilder.setAutoCancel(true).build());
        }
    }


    public int getRadiusValue() {
        SharedPreferences sp12 = getSharedPreferences("radius", MODE_PRIVATE);
        int radius = sp12.getInt("radius", 0);
        return radius;
    }


}