package com.juablaz.myplaces;

import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonArrayRequest;
import com.android.volley.toolbox.Volley;
import com.example.juancarlos.myplaces.R;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.firebase.messaging.FirebaseMessagingService;
import com.google.firebase.messaging.RemoteMessage;
import com.juablaz.myplaces.Util.CustomHurlStack;
import com.juablaz.myplaces.Util.CustomResponse;
import com.juablaz.myplaces.model.SharedMarker;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.UUID;

import static com.juablaz.myplaces.CoreMapActivity.sharedMarkers;
import static com.juablaz.myplaces.CoreMapActivity.userLogged;
import static com.juablaz.myplaces.Util.StringUtils.BAR;
import static com.juablaz.myplaces.Util.StringUtils.CALL_GETSHARED;
import static com.juablaz.myplaces.Util.StringUtils.DOUBLE_BAR;
import static com.juablaz.myplaces.Util.StringUtils.HOST;
import static com.juablaz.myplaces.Util.StringUtils.PORT;
import static com.juablaz.myplaces.Util.StringUtils.PROTOCOL;
import static com.juablaz.myplaces.Util.StringUtils.RESPONSE_OK;
import static com.juablaz.myplaces.Util.StringUtils.STATE_2xx;
import static com.juablaz.myplaces.Util.StringUtils.TWO_POINTS;
import static com.juablaz.myplaces.Util.StringUtils.UPDATED_USER_OK;

public class FirebaseMessageService extends FirebaseMessagingService implements Response.Listener<JSONArray>, Response.ErrorListener {

    public FirebaseMessageService() {
    }

    private static final String TAG = "MyFirebaseMsgService";
    String token;

    @Override
    public void onDeletedMessages() {

    }


    @Override
    public void onMessageReceived(RemoteMessage remoteMessage) {
        super.onMessageReceived(remoteMessage);

        Log.d("MyFirebaseMsgService", "GOT A MSSG");

        if (remoteMessage.getNotification() != null) {
            Log.e(TAG, "Title: " + remoteMessage.getNotification().getTitle());
            Log.e(TAG, "Body: " + remoteMessage.getNotification().getBody());
            Log.e(TAG, "NOTIF: " + remoteMessage.getNotification().getChannelId());
            callWebService("getSharedPlaces");

            addNotification();

        }
/*
        sendNotification(remoteMessage.getNotification().getTitle(),remoteMessage.getNotification().getBody());
*/
    }

    @Override
    public void onNewToken(String s) {
        super.onNewToken(s);
        Log.d("MyFirebaseMsgService", s);
        token = s;
        callWebService("update");
    }

    // Call My Web Service
    public void callWebService(String operation) {
        JSONArray jsonBody = new JSONArray();

        CustomHurlStack customHurlStack = new CustomHurlStack();
        RequestQueue requestQueue = Volley.newRequestQueue(this, customHurlStack);

        String url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/users";


        try {

            JSONObject userJSON = new JSONObject();
            userJSON.put("username", userLogged.getNombreUsuario());
            userJSON.put("myFirebaseToken", token);
            jsonBody.put(userJSON);

            Log.i(TAG, "###### SELECCIONANDO OPERACION #####");

            switch (operation) {
                case "update":
                    Log.i(TAG, "###### UPDATE #####");
                    break;

                case "getSharedPlaces":
                    Log.i(TAG, "###### GETSHAREDPLACES #####");
                    break;
            }


        } catch (JSONException e) {
            e.printStackTrace();
        }

        switch (operation) {
            case "remove":
                JsonArrayRequest jsonArrayRemoveRequest = new JsonArrayRequest(Request.Method.PUT, url, jsonBody, this, this);
                requestQueue.add(jsonArrayRemoveRequest);
                break;

            case "getSharedPlaces":
                url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/shared_places";
                String sharedUrl = String.format(url + "?username=%1$s", userLogged.getNombreUsuario());
                JsonArrayRequest jsonArrayGetSharedPlacesRequest = new JsonArrayRequest(sharedUrl, this, this);
                requestQueue.add(jsonArrayGetSharedPlacesRequest);
                break;

        }
    }


    @Override
    public void onErrorResponse(VolleyError error) {

        if (error != null) {

            Log.e(TAG, error.toString());
        }
    }


    @Override
    public void onResponse(JSONArray response) {
        if (response != null)
            Log.i("La respuesta es: ", response.toString());

        CustomResponse customResponse;
        if (response != null) {
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject = response.getJSONObject(0);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            customResponse = new CustomResponse(jsonObject.optString("status"),
                    jsonObject.optString("description"),
                    jsonObject.optString("message"),
                    jsonObject.optString("timeStamp"));
            if (jsonObject.optString("call") != null) {
                customResponse.setCall(jsonObject.optString("call"));
            }
            if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(UPDATED_USER_OK)) {
                Log.i(TAG, "token actualizado");
            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && !customResponse.getMessage().toString().isEmpty() && customResponse.getCall() != null && customResponse.getCall().equals(CALL_GETSHARED)) {
                JSONArray jsonGetAllArray = null;
                Log.i(TAG, customResponse.getMessage().toString());
                try {
                    jsonGetAllArray = new JSONArray(customResponse.getMessage().toString());
                } catch (JSONException e) {
                    e.printStackTrace();
                }

                if (jsonGetAllArray != null) {
                    sharedMarkers = new ArrayList<>();
                    for (int i = 0; i < jsonGetAllArray.length(); i++) {
                        try {
                            JSONObject jsonMarkerObject = jsonGetAllArray.getJSONObject(i);
                            LatLng latLng = new LatLng(Double.valueOf(jsonMarkerObject.getString("latitude")),
                                    Double.valueOf(jsonMarkerObject.getString("longitude")));

                            MarkerOptions options = new MarkerOptions()
                                    .snippet(jsonMarkerObject.getString("snippet"))
                                    .position(latLng)
                                    .draggable(true)
                                    .title(jsonMarkerObject.getString("title"));

                            SharedMarker sharedMarker = new SharedMarker();
                            sharedMarker.setMarkerOptions(options);
                            sharedMarker.setSentBy((jsonMarkerObject.getString("username")));

                            try {
                                sharedMarker.setSent(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss").parse(jsonMarkerObject.getString("date")));
                            } catch (ParseException e) {
                                e.printStackTrace();
                            }
                            sharedMarkers.add(sharedMarker);

                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }
                    // TODO: FIX THIS
                    Log.i(TAG, "COMPARTIDOS: " + sharedMarkers.size());
                    broadcastIntent();

                }

            } else {
                Log.i(TAG, "no ha ido las respuestas: " + sharedMarkers.size());

            }


        } else {
            Log.i(TAG, "COMPARTIDOS: " + sharedMarkers.size());

            Toast.makeText(this, "No se ha conectar con el servidor. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
        }


    }


    // Creates and displays a notification
    private void addNotification() {
        SharedPreferences sp = getSharedPreferences("notification", MODE_PRIVATE);
        boolean enabled = sp.getBoolean("notification", true);

        if (enabled) {
            Intent notificationIntent = new Intent(FirebaseMessageService.this, ShowMySharedPlacesActivity.class);

            notificationIntent.putExtra("isNotification", true);
            notificationIntent.putExtra("fromNotification", true);

            notificationIntent.setAction(Intent.ACTION_MAIN);
            notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);

            notificationIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_CLEAR_TASK);
            PendingIntent pIntent = PendingIntent.getActivity(FirebaseMessageService.this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);

            // Builds your notification
            NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(FirebaseMessageService.this, "default")
                    .setSmallIcon(R.mipmap.ic_launcher_round)
                    .setContentTitle("Lugares recibidos")
                    .setContentText("Te han compartido nuevos lugares")
                    .setVibrate(new long[]{1000, 1000, 1000, 1000, 1000})
                    .setContentIntent(pIntent)
                    .setPriority(NotificationCompat.PRIORITY_MAX);

            NotificationManager manager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

            manager.notify(UUID.randomUUID().hashCode(), notificationBuilder.setAutoCancel(true).build());

        }
    }

    public void broadcastIntent() {
        Log.i("HOLAA", "HOLAA");
        Intent intent = new Intent();
        intent.setAction("com.tutorialspoint.CUSTOM_INTENT");
        sendBroadcast(intent);
    }

}
