package com.juablaz.myplaces;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.View;
import android.widget.ImageButton;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonArrayRequest;
import com.android.volley.toolbox.Volley;
import com.example.juancarlos.myplaces.R;
import com.juablaz.myplaces.Util.CustomHurlStack;
import com.juablaz.myplaces.Util.CustomResponse;
import com.juablaz.myplaces.Util.RecyclerSharedViewAdapter;
import com.juablaz.myplaces.model.SharedMarker;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import static com.juablaz.myplaces.CoreMapActivity.sharedMarkers;
import static com.juablaz.myplaces.CoreMapActivity.userLogged;
import static com.juablaz.myplaces.Util.StringUtils.ADDED_SHARED_OK;
import static com.juablaz.myplaces.Util.StringUtils.BAR;
import static com.juablaz.myplaces.Util.StringUtils.DELETED_SHARED_OK;
import static com.juablaz.myplaces.Util.StringUtils.DOUBLE_BAR;
import static com.juablaz.myplaces.Util.StringUtils.HOST;
import static com.juablaz.myplaces.Util.StringUtils.PORT;
import static com.juablaz.myplaces.Util.StringUtils.PROTOCOL;
import static com.juablaz.myplaces.Util.StringUtils.RESPONSE_OK;
import static com.juablaz.myplaces.Util.StringUtils.STATE_2xx;
import static com.juablaz.myplaces.Util.StringUtils.TWO_POINTS;

public class ShowMySharedPlacesActivity extends AppCompatActivity implements Response.Listener<JSONArray>, Response.ErrorListener {

    public static List<SharedMarker> currentSelectedItems = new ArrayList<>();
    RecyclerSharedViewAdapter recyclerViewAdapter;
    RecyclerView recyclerView;
    boolean fromNotification;
    private BroadcastReceiver mReceiver = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_show_my_shared_places);

        recyclerView = findViewById(R.id.recycler_view);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        recyclerViewAdapter = new RecyclerSharedViewAdapter(ShowMySharedPlacesActivity.this, sharedMarkers);

        recyclerView.setAdapter(recyclerViewAdapter);

        mReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                recyclerView.setAdapter(new RecyclerSharedViewAdapter(ShowMySharedPlacesActivity.this, sharedMarkers));
                recyclerView.invalidate();
            }
        };


        ImageButton deleteButton = findViewById(R.id.delete);
        deleteButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(final View view) {

                currentSelectedItems = recyclerViewAdapter.getSharedMarkerList();
                if (!currentSelectedItems.isEmpty()) {
                    callWebService("remove");
                } else {
                    Toast.makeText(ShowMySharedPlacesActivity.this, "Debe seleccionar algún lugar para rechazar.", Toast.LENGTH_LONG).show();
                }


            }
        });

        ImageButton addButton = findViewById(R.id.add);
        addButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                currentSelectedItems = recyclerViewAdapter.getSharedMarkerList();



                if (!currentSelectedItems.isEmpty()) {
                    callWebService("add");
                } else {
                    Toast.makeText(ShowMySharedPlacesActivity.this, "Debe seleccionar algún lugar para aceptar.", Toast.LENGTH_LONG).show();
                }

                Log.i("###SHOW", currentSelectedItems.size() + "");


            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        IntentFilter filter = new IntentFilter("com.tutorialspoint.CUSTOM_INTENT");
        registerReceiver(mReceiver, filter);
    }

    @Override
    public void onErrorResponse(VolleyError error) {
        Toast.makeText(this, "Usuario no encontrado.", Toast.LENGTH_LONG).show();

    }

    @Override
    public void onResponse(JSONArray response) {
        if (response != null)
            Log.i("La respuesta es: ", response.toString());

        CustomResponse customResponse;
        if (response != null) {
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject = response.getJSONObject(0);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            customResponse = new CustomResponse(jsonObject.optString("status"),
                    jsonObject.optString("description"),
                    jsonObject.optString("message"),
                    jsonObject.optString("timeStamp"));

            if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(DELETED_SHARED_OK)) {

                //Elimino esos lugares del total de lugares.
                sharedMarkers.removeAll(currentSelectedItems);

                Toast.makeText(this, "Lugares eliminados correctamente.", Toast.LENGTH_LONG).show();
                recyclerView.setAdapter(new RecyclerSharedViewAdapter(ShowMySharedPlacesActivity.this, sharedMarkers));
                recyclerView.invalidate();

            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(ADDED_SHARED_OK)) {
                Log.i("HOLA", "HOLaaaaaaaaA");

               /* Intent resultIntent = new Intent();
                setResult(Activity.RESULT_OK, resultIntent);
                finish();
*/

                    Intent intentRegister = new Intent(ShowMySharedPlacesActivity.this, CoreMapActivity.class);
                    intentRegister.putExtra("fromNotifToCore", true);

                    ShowMySharedPlacesActivity.this.startActivity(intentRegister);




                /*  Toast.makeText(this, "Lugares compartidos correctamente.", Toast.LENGTH_LONG).show();
                Intent coreMap = new Intent(ShowPlacesActivity.this, CoreMapActivity.class);
                ShowPlacesActivity.this.startActivity(coreMap);*/
            }

        } else {
            Toast.makeText(this, "No se ha conectar con el servidor. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
        }
    }

    // Call My Web Service
    private void callWebService(String operation) {
        JSONArray jsonBody = new JSONArray();

        CustomHurlStack customHurlStack = new CustomHurlStack();
        RequestQueue requestQueue = Volley.newRequestQueue(this, customHurlStack);

        String url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/shared_places";

        try {

            JSONObject userJSON = new JSONObject();
            userJSON.put("username", userLogged.getNombreUsuario());
            jsonBody.put(userJSON);

            JSONArray markers = new JSONArray();
            Log.i("###########", "###### SELECCIONANDO OPERACION #####");

            switch (operation) {
                case "remove":
                    Log.i("###########", "###### REMOVE #####");

                    //LLamada para borrar
                    for (SharedMarker sharedMarker : currentSelectedItems) {

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("latitude", sharedMarker.getMarkerOptions().getPosition().latitude);
                        markerJSON.put("longitude", sharedMarker.getMarkerOptions().getPosition().longitude);
                        markerJSON.put("user_sharing", sharedMarker.getSentBy());
                        markers.put(markerJSON);
                    }
                    jsonBody.put(markers);

                    break;

                case "add":
                    Log.i("###########", "###### ADD #####");
                    for (SharedMarker sharedMarker : currentSelectedItems) {
                        Log.i("###SHOW", "He entrado a por uno: " + sharedMarker.toString());

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("latitude", sharedMarker.getMarkerOptions().getPosition().latitude);
                        markerJSON.put("longitude", sharedMarker.getMarkerOptions().getPosition().longitude);
                        markerJSON.put("user_sharing", sharedMarker.getSentBy());
                        markers.put(markerJSON);
                    }
                    jsonBody.put(markers);

                    break;
            }

            Log.i("###SHOW", jsonBody.toString());

        } catch (JSONException e) {
            e.printStackTrace();
        }

        switch (operation) {
            case "remove":
                JsonArrayRequest jsonArrayRemoveRequest = new JsonArrayRequest(Request.Method.DELETE, url, jsonBody, this, this);
                requestQueue.add(jsonArrayRemoveRequest);
                break;
            case "add":
                JsonArrayRequest jsonArrayAddRequest = new JsonArrayRequest(Request.Method.POST, url, jsonBody, this, this);
                requestQueue.add(jsonArrayAddRequest);
                break;
        }
    }

    @Override
    public void onBackPressed() {
        Intent intentRegister = new Intent(ShowMySharedPlacesActivity.this, CoreMapActivity.class);

        ShowMySharedPlacesActivity.this.startActivity(intentRegister);

    }

    @Override
    public void onStop() {
        super.onStop();

        unregisterReceiver(mReceiver);
    }
}
