package com.juablaz.myplaces;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonArrayRequest;
import com.android.volley.toolbox.Volley;
import com.example.juancarlos.myplaces.R;
import com.google.android.gms.maps.model.Marker;
import com.juablaz.myplaces.Util.CustomHurlStack;
import com.juablaz.myplaces.Util.CustomResponse;
import com.juablaz.myplaces.Util.RecyclerViewAdapter;
import com.juablaz.myplaces.model.CustomMarker;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import static com.juablaz.myplaces.CoreMapActivity.userLogged;
import static com.juablaz.myplaces.CoreMapActivity.visibleMarkers;
import static com.juablaz.myplaces.Util.StringUtils.BAR;
import static com.juablaz.myplaces.Util.StringUtils.DELETED_OK;
import static com.juablaz.myplaces.Util.StringUtils.DOUBLE_BAR;
import static com.juablaz.myplaces.Util.StringUtils.HOST;
import static com.juablaz.myplaces.Util.StringUtils.PORT;
import static com.juablaz.myplaces.Util.StringUtils.PROTOCOL;
import static com.juablaz.myplaces.Util.StringUtils.RESPONSE_OK;
import static com.juablaz.myplaces.Util.StringUtils.SHARED_OK;
import static com.juablaz.myplaces.Util.StringUtils.STATE_2xx;
import static com.juablaz.myplaces.Util.StringUtils.TWO_POINTS;

public class ShowPlacesActivity extends AppCompatActivity implements Response.Listener<JSONArray>, Response.ErrorListener {

    private List<CustomMarker> currentSelectedItems = new ArrayList<>();
    RecyclerViewAdapter recyclerViewAdapter;
    TextView userTextInput;
    TextInputLayout inputLayoutUserName;
    Dialog dialog;
    String userInput;
    RecyclerView recyclerView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_show_places);

        recyclerView = findViewById(R.id.recycler_view);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        recyclerViewAdapter = new RecyclerViewAdapter(ShowPlacesActivity.this, visibleMarkers);

        recyclerView.setAdapter(recyclerViewAdapter);

        ImageButton deleteButton = findViewById(R.id.delete);
        deleteButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(final View view) {
                currentSelectedItems = recyclerViewAdapter.getMarkerList();
                if (!currentSelectedItems.isEmpty()) {
                    callWebService("remove");
                } else {
                    Toast.makeText(ShowPlacesActivity.this, "Debe seleccionar algún lugar para borrar.", Toast.LENGTH_LONG).show();
                }
            }
        });

        ImageButton shareButton = findViewById(R.id.share);
        shareButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                currentSelectedItems = recyclerViewAdapter.getMarkerList();


                if (!currentSelectedItems.isEmpty()) {
                    //set up dialog
                    dialog = new Dialog(ShowPlacesActivity.this);
                    dialog.setContentView(R.layout.activity_pop_up_user);
                    dialog.setCancelable(true);
                    dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
                    //there are a lot of settings, for dialog, check them all out!

                    //set up userTextInput
                    userTextInput = dialog.findViewById(R.id.input_user);
                    inputLayoutUserName = dialog.findViewById(R.id.input_layout_name);

                    //set up button
                    Button button = dialog.findViewById(R.id.save);
                    button.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            if (userTextInput.getText().toString().isEmpty()) {
                                inputLayoutUserName.setError("Debe introducir un nombre de usuario");
                            } else {
                                inputLayoutUserName.setErrorEnabled(false);
                                //CallWebService
                                userInput = userTextInput.getText().toString();

                                //TODO FIX THIS
                               /* if (userInput.equals(userLogged.getNombreUsuario())) {
                                    inputLayoutUserName.setError("No puede compartir consigo mismo");

                                } else {*/
                                    callWebService("share");
                                    dialog.dismiss();
                            /*    }*/

                            }

                            //Log.i("tag", userInput);
                        }
                    });
                    //now that the dialog is set up, it's time to show it
                    dialog.show();
                } else {
                    Toast.makeText(ShowPlacesActivity.this, "Debe seleccionar algún lugar para compartir.", Toast.LENGTH_LONG).show();
                }
            }
        });
    }

    @Override
    public void onErrorResponse(VolleyError error) {
        Toast.makeText(this, "Usuario no encontrado. Compruebe el nombre de usuario introducido.", Toast.LENGTH_LONG).show();

    }

    @Override
    public void onResponse(JSONArray response) {
        if (response != null)
            Log.i("La respuesta es: ", response.toString());

        CustomResponse customResponse;
        if (response != null) {
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject = response.getJSONObject(0);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            customResponse = new CustomResponse(jsonObject.optString("status"),
                    jsonObject.optString("description"),
                    jsonObject.optString("message"),
                    jsonObject.optString("timeStamp"));

            if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(DELETED_OK)) {
                for (CustomMarker customMarker : currentSelectedItems) {
                    Marker marker = customMarker.getMarker();
                    Log.i("MARKER BORRADO: ", marker.getTitle() + " " + marker.getSnippet());
                    marker.remove();
                }
                //Elimino esos lugares del total de lugares.
                if (!currentSelectedItems.isEmpty()) {
                    visibleMarkers.removeAll(currentSelectedItems);

                    Toast.makeText(this, "Lugares eliminados correctamente.", Toast.LENGTH_LONG).show();
                }
            /*    Intent coreMap = new Intent(ShowPlacesActivity.this, CoreMapActivity.class);
                ShowPlacesActivity.this.startActivity(coreMap);*/
                recyclerView.setAdapter(new RecyclerViewAdapter(ShowPlacesActivity.this, visibleMarkers));
                recyclerView.invalidate();
            } else if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(SHARED_OK)) {
                Toast.makeText(this, "Lugares compartidos correctamente.", Toast.LENGTH_LONG).show();
             /*   Intent coreMap = new Intent(ShowPlacesActivity.this, CoreMapActivity.class);
                ShowPlacesActivity.this.startActivity(coreMap);*/
            }

        } else {
            Toast.makeText(this, "No se ha conectar con el servidor. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
        }
    }

    // Call My Web Service
    private void callWebService(String operation) {
        JSONArray jsonBody = new JSONArray();

        CustomHurlStack customHurlStack = new CustomHurlStack();
        RequestQueue requestQueue = Volley.newRequestQueue(this, customHurlStack);

        String url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/places";


        try {

            JSONObject userJSON = new JSONObject();
            JSONObject targetUserJSON = new JSONObject();
            userJSON.put("username", userLogged.getNombreUsuario());
            jsonBody.put(userJSON);

            JSONArray markers = new JSONArray();
            Log.i("###########", "###### SELECCIONANDO OPERACION #####");

            switch (operation) {
                case "remove":
                    Log.i("###########", "###### REMOVE #####");

                    //LLamada para borrar
                    for (CustomMarker customMarker : currentSelectedItems) {
                        Marker marker = customMarker.getMarker();

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("latitude", marker.getPosition().latitude);
                        markerJSON.put("longitude", marker.getPosition().longitude);
                        markers.put(markerJSON);
                    }
                    jsonBody.put(markers);

                    break;

                case "share":
                    url = url + "/share";
                    Log.i("###########", "###### SHARE #####");
                    for (CustomMarker customMarker : currentSelectedItems) {
                        Marker marker = customMarker.getMarker();

                        JSONObject markerJSON = new JSONObject();
                        markerJSON.put("latitude", marker.getPosition().latitude);
                        markerJSON.put("longitude", marker.getPosition().longitude);
                        markerJSON.put("title", marker.getTitle());
                        markerJSON.put("snippet", marker.getSnippet());
                        markers.put(markerJSON);
                    }
                    jsonBody.put(markers);
                    targetUserJSON.put("target_username", userInput);
                    jsonBody.put(targetUserJSON);

                    break;
            }

            Log.i("###SHOW", jsonBody.toString());

        } catch (JSONException e) {
            e.printStackTrace();
        }

        switch (operation) {
            case "remove":
                JsonArrayRequest jsonArrayRemoveRequest = new JsonArrayRequest(Request.Method.DELETE, url, jsonBody, this, this);
                requestQueue.add(jsonArrayRemoveRequest);
                break;
            case "share":
                JsonArrayRequest jsonArrayShareRequest = new JsonArrayRequest(Request.Method.POST, url, jsonBody, this, this);
                requestQueue.add(jsonArrayShareRequest);
                break;
        }
    }
}
