package com.juablaz.myplaces;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.text.method.PasswordTransformationMethod;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;
import com.example.juancarlos.myplaces.R;
import com.juablaz.myplaces.Util.CustomResponse;
import com.juablaz.myplaces.model.Usuario;

import org.json.JSONException;
import org.json.JSONObject;

import static com.juablaz.myplaces.Util.StringUtils.BAR;
import static com.juablaz.myplaces.Util.StringUtils.DOUBLE_BAR;
import static com.juablaz.myplaces.Util.StringUtils.HOST;
import static com.juablaz.myplaces.Util.StringUtils.PORT;
import static com.juablaz.myplaces.Util.StringUtils.PROTOCOL;
import static com.juablaz.myplaces.Util.StringUtils.RESPONSE_OK;
import static com.juablaz.myplaces.Util.StringUtils.SING_UP_OK;
import static com.juablaz.myplaces.Util.StringUtils.STATE_2xx;
import static com.juablaz.myplaces.Util.StringUtils.STATE_409;
import static com.juablaz.myplaces.Util.StringUtils.TWO_POINTS;


public class SignupActivity extends AppCompatActivity implements Response.Listener<JSONObject>, Response.ErrorListener {

    TextView userName;
    TextView password;
    TextView email;

    Button singup;
    TextView login;
    Usuario userLogged;

    TextInputLayout inputLayoutUserName, inputLayoutUserPassword, inputLayoutEmail;

    RequestQueue requestQueue;
    JsonObjectRequest jsonObjectRequest;

    @SuppressLint("ClickableViewAccessibility")
    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_signup);

        userName = findViewById(R.id.input_nombreUsuario);
        password = findViewById(R.id.input_contrasena);
        email = findViewById(R.id.input_correo);

        singup = findViewById(R.id.btn_crearCuenta);
        login = findViewById(R.id.link_iniciar);

        inputLayoutUserName = findViewById(R.id.input_layout_name);
        inputLayoutUserPassword = findViewById(R.id.input_layout_contrasena);
        inputLayoutEmail = findViewById(R.id.input_layout_correo);

        password.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                final int DRAWABLE_RIGHT = 2;

                if (event.getAction() == MotionEvent.ACTION_UP) {
                    if (event.getRawX() >= (password.getRight() - password.getCompoundDrawables()[DRAWABLE_RIGHT].getBounds().width())) {
                        if (password.getTransformationMethod() != null) { //No se muestra, quiero mostrarla
                            password.setTransformationMethod(null);
                            password.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_password, 0, R.drawable.ic_image_enabled, 0);

                        } else { //Se muestra, quiero ocultarla
                            password.setTransformationMethod(new PasswordTransformationMethod());
                            password.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_password, 0, R.drawable.ic_image_disabled, 0);
                        }
                        return true;
                    }
                }
                return false;
            }
        });

        singup.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                singup.setEnabled(false);
                boolean canCall = true;
                //Enviamos la petición
                if (userName.getText().toString().isEmpty()) {
                    inputLayoutUserName.setError("Debe introducir un nombre de usuario");
                    canCall = false;
                } else {
                    inputLayoutUserName.setErrorEnabled(false);
                }
                if (password.getText().toString().isEmpty()) {
                    inputLayoutUserPassword.setError("Debe introducir una contraseña");
                    canCall = false;
                } else {
                    inputLayoutUserPassword.setErrorEnabled(false);
                }
                if (email.getText().toString().isEmpty() || !isValidEmail(email.getText().toString())) {
                    inputLayoutEmail.setError("Debe introducir un email válido");
                    canCall = false;
                } else {
                    inputLayoutEmail.setErrorEnabled(false);
                }
                if (canCall) {
                    callWebService();
                }

                singup.setEnabled(true);
            }
        });

        login.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intentRegister = new Intent(SignupActivity.this, LoginActivity.class);
                SignupActivity.this.startActivity(intentRegister);
            }
        });

    }

    private void callWebService() {
        JSONObject jsonBody = new JSONObject();
        requestQueue = Volley.newRequestQueue(this);

        String url = PROTOCOL + TWO_POINTS + DOUBLE_BAR + HOST + TWO_POINTS + PORT + BAR + "myplaces/sing-up";

        try {
            Log.i("DATOS DE USUARIO", userName.getText().toString() + " " + email.getText().toString() + " " + password.getText().toString() + " ");

            jsonBody = new JSONObject();
            jsonBody.put("username", userName.getText().toString());
            jsonBody.put("email", email.getText().toString());
            jsonBody.put("password", password.getText().toString());

        } catch (JSONException e) {
            e.printStackTrace();
        }
        Log.i("JSON", jsonBody.toString());
        jsonObjectRequest = new JsonObjectRequest(Request.Method.POST, url, jsonBody, this, this);

        requestQueue.add(jsonObjectRequest);
    }


    @Override
    public void onErrorResponse(VolleyError error) {
        switch (error.networkResponse.statusCode) {
            case STATE_409:
                Toast.makeText(this, "El usuario introducido ya existe. Inténtelo con otro diferente.", Toast.LENGTH_LONG).show();
        }

        Log.i("ERROR", error.toString());
    }

    @Override
    public void onResponse(JSONObject response) {
        userLogged = new Usuario();
        if (response != null)
            Log.i("La respuesta es: ", response.toString());
        CustomResponse customResponse = null;
        if (response != null) {
            customResponse = new CustomResponse(response.optString("status"),
                    response.optString("description"),
                    response.optString("message"),
                    response.optString("timeStamp"));
        } else {
            Toast.makeText(this, "No se ha podido registrar. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
        }

        if (customResponse != null) {
            if (customResponse.getStatus().equals(STATE_2xx) && customResponse.getDescription().equals(RESPONSE_OK) && customResponse.getMessage().toString().equals(SING_UP_OK)) {
                userLogged.setNombreUsuario(userName.getText().toString());
                userLogged.setContrasena(password.getText().toString());

                Intent intentRegister = new Intent(SignupActivity.this, LoginActivity.class);
                Toast.makeText(this, "Usuario creado correctamente. Inicie sesión.", Toast.LENGTH_LONG).show();

                SignupActivity.this.startActivity(intentRegister);
            } else {
                Toast.makeText(this, "Error, no se ha podido registrar.", Toast.LENGTH_LONG).show();
            }
        } else {
            Toast.makeText(this, "No se ha podido registrar. Inténtelo de nuevo.", Toast.LENGTH_LONG).show();
        }
    }

    public static boolean isValidEmail(CharSequence target) {
        if (TextUtils.isEmpty(target)) {
            return false;
        } else {
            return android.util.Patterns.EMAIL_ADDRESS.matcher(target).matches();
        }
    }

    @Override
    public void onBackPressed() {

        //Do nothing
    }
}
