package com.juablaz.myplaces.Util;

import android.content.Context;
import android.content.Intent;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.TextView;

import com.example.juancarlos.myplaces.R;
import com.juablaz.myplaces.ViewPlaceActivity;
import com.juablaz.myplaces.model.CustomMarker;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import static com.juablaz.myplaces.CoreMapActivity.currentLocation;


public class RecyclerViewAdapter extends RecyclerView.Adapter<RecyclerViewAdapter.ViewHolder> {

    private Context context;
    private List<CustomMarker> list;

    public RecyclerViewAdapter(Context context, List<CustomMarker> list) {
        this.context = context;
        this.list = list;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.model_card_view, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final ViewHolder holder, final int position) {

        CustomMarker customMarker = list.get(position);

        holder.tv_title.setText(customMarker.getMarker().getTitle());
        holder.tv_description.setText(customMarker.getMarker().getSnippet());

        DecimalFormat df = new DecimalFormat("###.##");

        holder.tv_distance.setText("Distancia: " + df.format(customMarker.getDistancia() / 1000) + " Kms");

        holder.checkBox.setChecked(customMarker.isSelected());
        holder.checkBox.setTag(list.get(position));

        holder.checkBox.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                CustomMarker customMarker1 = (CustomMarker) holder.checkBox.getTag();

                customMarker1.setSelected(holder.checkBox.isChecked());

                list.get(position).setSelected(holder.checkBox.isChecked());

            }
        });

        holder.tv_title.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent edit = new Intent(context, ViewPlaceActivity.class);

                edit.putExtra("currentLocationLatitude", currentLocation.getLatitude());
                edit.putExtra("currentLocationLongitude", currentLocation.getLongitude());
                edit.putExtra("title", holder.tv_title.getText());
                edit.putExtra("snippet", holder.tv_description.getText());
                edit.putExtra("isNotification", false);

                context.startActivity(edit);
            }
        });
        holder.tv_distance.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent edit = new Intent(context, ViewPlaceActivity.class);

                edit.putExtra("currentLocationLatitude", currentLocation.getLatitude());
                edit.putExtra("currentLocationLongitude", currentLocation.getLongitude());
                edit.putExtra("title", holder.tv_title.getText());
                edit.putExtra("snippet", holder.tv_description.getText());
                edit.putExtra("isNotification", false);

                context.startActivity(edit);
            }
        });
        holder.tv_description.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent edit = new Intent(context, ViewPlaceActivity.class);

                edit.putExtra("currentLocationLatitude", currentLocation.getLatitude());
                edit.putExtra("currentLocationLongitude", currentLocation.getLongitude());
                edit.putExtra("title", holder.tv_title.getText());
                edit.putExtra("snippet", holder.tv_description.getText());
                edit.putExtra("isNotification", false);

                context.startActivity(edit);
            }
        });
    }

    @Override
    public int getItemCount() {
        return list.size();
    }

    public static class ViewHolder extends RecyclerView.ViewHolder {

        TextView tv_title, tv_description, tv_distance;
        CheckBox checkBox;

        public ViewHolder(View itemView) {
            super(itemView);

            tv_title = itemView.findViewById(R.id.title_view);
            tv_description = itemView.findViewById(R.id.description_view);
            tv_distance = itemView.findViewById(R.id.distance_view);
            checkBox = itemView.findViewById(R.id.checkbox_view);
        }
    }

    public List<CustomMarker> getMarkerList() {
        List<CustomMarker> customMarkers = new ArrayList<>();
        for (CustomMarker cm : list) {
            if (cm.isSelected()) {
                customMarkers.add(cm);
            }
        }
        return customMarkers;
    }
}